@@ -42,9 +42,6 @@ namespace awkward {
     void
       clear() override;
 
-    const ContentPtr
-      snapshot() const override;
-
     /// @copydoc Builder::active()
     ///
     /// A BoolBuilder is never active.
@@ -66,6 +63,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -96,6 +99,11 @@ namespace awkward {
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
+    const GrowableBuffer<uint8_t>& buffer() const { return buffer_; }
+
   private:
     const ArrayBuilderOptions options_;
     GrowableBuffer<uint8_t> buffer_;
