@@ -8,8 +8,6 @@
 #include <vector>
 
 #include "awkward/common.h"
-#include "awkward/Content.h"
-#include "awkward/type/Type.h"
 
 namespace awkward {
   class Content;
@@ -41,18 +39,6 @@ namespace awkward {
     virtual void
       clear() = 0;
 
-    /// @brief Turns the accumulated data into a Content array.
-    ///
-    /// This operation only converts Builder nodes into Content nodes; the
-    /// buffers holding array data are shared between the Builder and the
-    /// Content. Hence, taking a snapshot is a constant-time operation.
-    ///
-    /// It is safe to take multiple snapshots while accumulating data. The
-    /// shared buffers are only appended to, which affects elements beyond
-    /// the limited view of old snapshots.
-    virtual const ContentPtr
-      snapshot() const = 0;
-
     /// @brief If `true`, this node has started but has not finished a
     /// multi-step command (e.g. `beginX ... endX`).
     virtual bool
@@ -78,6 +64,14 @@ namespace awkward {
     virtual const BuilderPtr
       complex(std::complex<double> x) = 0;
 
+    /// @brief Adds a datetime value `x` to the accumulated data.
+    virtual const BuilderPtr
+      datetime(int64_t x, const std::string& unit) = 0;
+
+    /// @brief Adds a timedelta value `x` to the accumulated data.
+    virtual const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) = 0;
+
     /// @brief Adds a string value `x` with a given `length` and `encoding`
     /// to the accumulated data.
     ///
