@@ -53,9 +53,6 @@ namespace awkward {
     void
       clear() override;
 
-    const ContentPtr
-      snapshot() const override;
-
     /// @copydoc Builder::active()
     ///
     /// A Float64Builder is never active.
@@ -77,6 +74,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -107,6 +110,9 @@ namespace awkward {
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
   private:
     const ArrayBuilderOptions options_;
     GrowableBuffer<double> buffer_;
