@@ -37,6 +37,7 @@ namespace awkward {
                    const std::shared_ptr<T>& array,
                    bool hasnull);
 
+    // FIXME: refactor
     /// @brief Raw pointer to the original Content `array`.
     const Content*
       arrayptr() const;
@@ -66,6 +67,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -93,6 +100,17 @@ namespace awkward {
     const BuilderPtr
       endrecord() override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
+    const GrowableBuffer<int64_t>& buffer() const { return index_; }
+    GrowableBuffer<int64_t>& index() { return index_; }
+
+    bool hasnull() const { return hasnull_; }
+
+    // FIXME: refactor
+    const std::shared_ptr<T> array() const { return array_; }
+
   protected:
     const ArrayBuilderOptions options_;
     GrowableBuffer<int64_t> index_;
@@ -116,11 +134,10 @@ namespace awkward {
     const std::string
       classname() const override;
 
-    const ContentPtr
-      snapshot() const override;
-
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
+
+    const std::shared_ptr<Content> array() const { return array_; }
   };
 
   class IndexedI32Builder: public IndexedBuilder<IndexedArray32> {
@@ -134,9 +151,6 @@ namespace awkward {
     const std::string
       classname() const override;
 
-    const ContentPtr
-      snapshot() const override;
-
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
   };
@@ -152,9 +166,6 @@ namespace awkward {
     const std::string
       classname() const override;
 
-    const ContentPtr
-      snapshot() const override;
-
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
   };
@@ -170,9 +181,6 @@ namespace awkward {
     const std::string
       classname() const override;
 
-    const ContentPtr
-      snapshot() const override;
-
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
   };
@@ -188,9 +196,6 @@ namespace awkward {
     const std::string
       classname() const override;
 
-    const ContentPtr
-      snapshot() const override;
-
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
   };
@@ -206,9 +211,6 @@ namespace awkward {
     const std::string
       classname() const override;
 
-    const ContentPtr
-      snapshot() const override;
-
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
   };
