@@ -48,9 +48,6 @@ namespace awkward {
     void
       clear() override;
 
-    const ContentPtr
-      snapshot() const override;
-
     /// @copydoc Builder::active()
     ///
     /// Calling #beginlist makes a ListBuilder active; #endlist makes it
@@ -73,6 +70,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -103,12 +106,22 @@ namespace awkward {
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
+    const GrowableBuffer<int64_t>& buffer() const { return offsets_; }
+
+    const BuilderPtr builder() const { return content_; }
+
+    bool begun() { return begun_; }
+
   private:
     const ArrayBuilderOptions options_;
     GrowableBuffer<int64_t> offsets_;
     BuilderPtr content_;
     bool begun_;
 
+  public:
     void
       maybeupdate(const BuilderPtr& tmp);
   };
