@@ -66,9 +66,6 @@ namespace awkward {
     void
       clear() override;
 
-    const ContentPtr
-      snapshot() const override;
-
     /// @copydoc Builder::active()
     ///
     /// Calling #beginrecord makes a RecordBuilder active; #endrecord makes it
@@ -91,6 +88,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -121,6 +124,17 @@ namespace awkward {
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
+    const std::vector<std::string>& keys() const { return keys_; }
+
+    const std::vector<BuilderPtr>& builders() const { return contents_; }
+
+    bool begun() { return begun_; }
+
+    int64_t nextindex() { return nextindex_; }
+
   private:
     const BuilderPtr
       field_fast(const char* key);
@@ -138,7 +152,9 @@ namespace awkward {
     bool begun_;
     int64_t nextindex_;
     int64_t nexttotry_;
+    int64_t keys_size_;
 
+  public:
     void
       maybeupdate(int64_t i, const BuilderPtr& tmp);
   };
