@@ -56,9 +56,6 @@ namespace awkward {
     void
       clear() override;
 
-    const ContentPtr
-      snapshot() const override;
-
     /// @copydoc Builder::active()
     ///
     /// A StringBuilder is never active.
@@ -80,6 +77,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -110,6 +113,13 @@ namespace awkward {
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
+    const GrowableBuffer<int64_t>& buffer() const { return offsets_; }
+
+    const GrowableBuffer<uint8_t>& content() const { return content_; }
+
   private:
     const ArrayBuilderOptions options_;
     GrowableBuffer<int64_t> offsets_;
