@@ -52,9 +52,6 @@ namespace awkward {
     void
       clear() override;
 
-    const ContentPtr
-      snapshot() const override;
-
     /// @copydoc Builder::active()
     ///
     /// Calling #begintuple makes a TupleBuilder active; #endtuple makes it
@@ -77,6 +74,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -107,6 +110,15 @@ namespace awkward {
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
+    const std::vector<BuilderPtr>& contents() const { return contents_; }
+
+    bool begun() { return begun_; }
+
+    int64_t nextindex() { return nextindex_; }
+
   private:
     const ArrayBuilderOptions options_;
     std::vector<BuilderPtr> contents_;
@@ -114,6 +126,7 @@ namespace awkward {
     bool begun_;
     int64_t nextindex_;
 
+  public:
     void
       maybeupdate(int64_t i, const BuilderPtr& tmp);
   };
