@@ -47,9 +47,6 @@ namespace awkward {
     void
       clear() override;
 
-    const ContentPtr
-      snapshot() const override;
-
     /// @copydoc Builder::active()
     ///
     /// A UnionBuilder is active if and only if one of its `contents` is
@@ -72,6 +69,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -102,6 +105,20 @@ namespace awkward {
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
+    const GrowableBuffer<int8_t>& tags() const {  return tags_; }
+    GrowableBuffer<int8_t>& tags_buffer() {  return tags_; }
+
+    const GrowableBuffer<int64_t>& index() const { return index_; }
+    GrowableBuffer<int64_t>& index_buffer() { return index_; }
+
+    const std::vector<BuilderPtr>& contents() const { return contents_; }
+    std::vector<BuilderPtr>& builders() { return contents_; }
+
+    int8_t current() { return current_;}
+
   private:
     const ArrayBuilderOptions options_;
     GrowableBuffer<int8_t> tags_;
