@@ -39,9 +39,6 @@ namespace awkward {
     void
       clear() override;
 
-    const ContentPtr
-      snapshot() const override;
-
     /// @copydoc Builder::active()
     ///
     /// An UnknownBuilder is never active.
@@ -63,6 +60,12 @@ namespace awkward {
     const BuilderPtr
       complex(std::complex<double> x) override;
 
+    const BuilderPtr
+      datetime(int64_t x, const std::string& unit) override;
+
+    const BuilderPtr
+      timedelta(int64_t x, const std::string& unit) override;
+
     const BuilderPtr
       string(const char* x, int64_t length, const char* encoding) override;
 
@@ -93,6 +96,11 @@ namespace awkward {
     const BuilderPtr
       append(const ContentPtr& array, int64_t at) override;
 
+    const ArrayBuilderOptions&
+      options() const { return options_; }
+
+    const int64_t nullcount() const { return nullcount_; }
+
   private:
     const ArrayBuilderOptions options_;
     int64_t nullcount_;
