@@ -41,8 +41,8 @@ namespace awkward {
         complex64,
         complex128,
         complex256,
-        // datetime64,
-        // timedelta64,
+        datetime64,
+        timedelta64,
         size
     };
 
@@ -60,7 +60,7 @@ namespace awkward {
 
     /// @brief Convert a dtype enum into a NumPy format string.
     const std::string
-    dtype_to_format(dtype dt);
+    dtype_to_format(dtype dt, const std::string& format = "");
 
     /// @brief Convert a dtype enum into an itemsize.
     int64_t
@@ -164,6 +164,11 @@ namespace awkward {
 
     using Parameters = std::map<std::string, std::string>;
 
+    /// @brief Returns `true` if `myvalue` is equal to `value` when interpreted
+    /// as JSON.
+    bool
+    json_equals(const std::string &myvalue, const std::string &value);
+
     /// @brief Returns `true` if the value associated with a `key` in
     /// `parameters` is equal to the specified `value`.
     ///
