@@ -927,7 +927,7 @@ class type_getattr_record(numba.core.typing.templates.AttributeTemplate):
 
                 return numba.types.BoundFunction(type_method, recordviewtype)
 
-        for attrname, typer, lower in ak._util.numba_attrs(
+        for attrname, typer, _ in ak._util.numba_attrs(
             recordviewtype.arrayviewtype.type, recordviewtype.arrayviewtype.behavior
         ):
             if attr == attrname:
@@ -964,9 +964,11 @@ def register_unary_operator(unaryop):
                     left = args[0].arrayviewtype.type
                     behavior = args[0].arrayviewtype.behavior
 
-                for typer, lower in ak._util.numba_unaryops(unaryop, left, behavior):
-                    numba.extending.lower_builtin(unaryop, *args)(lower)
-                    return typer(unaryop, args[0])
+                    for typer, lower in ak._util.numba_unaryops(
+                        unaryop, left, behavior
+                    ):
+                        numba.extending.lower_builtin(unaryop, *args)(lower)
+                        return typer(unaryop, args[0])
 
 
 for unaryop in (
@@ -997,9 +999,12 @@ def register_binary_operator(binop):
                     if behavior is None:
                         behavior = args[1].arrayviewtype.behavior
 
-                for typer, lower in ak._util.numba_binops(binop, left, right, behavior):
-                    numba.extending.lower_builtin(binop, *args)(lower)
-                    return typer(binop, args[0], args[1])
+                if left is not None or right is not None:
+                    for typer, lower in ak._util.numba_binops(
+                        binop, left, right, behavior
+                    ):
+                        numba.extending.lower_builtin(binop, *args)(lower)
+                        return typer(binop, args[0], args[1])
 
 
 for binop in (
@@ -1297,7 +1302,7 @@ def typeof_PartitionedView(obj, c):
     return PartitionedViewType(obj.type, obj.behavior, obj.fields)
 
 
-class PartitionedViewType(numba.types.Type):
+class PartitionedViewType(numba.types.IterableType, numba.types.Sized):
     stopstype = numba.types.Array(numba.intp, 1, "C")
 
     def __init__(self, type, behavior, fields):
@@ -1312,6 +1317,10 @@ class PartitionedViewType(numba.types.Type):
         self.behavior = behavior
         self.fields = fields
 
+    @property
+    def iterator_type(self):
+        return PartitionedIteratorType(self)
+
     def toArrayViewType(self):
         return ArrayViewType(self.type, self.behavior, self.fields)
 
