@@ -507,7 +507,7 @@ class NumpyArrayType(ContentType):
 
     @classmethod
     def tolookup(cls, layout, positions, sharedptrs, arrays):
-        array = ak.nplike.of(layout).asarray(layout)
+        array = ak.nplike.of(layout).ascontiguousarray(layout)
         assert len(array.shape) == 1
         pos = len(positions)
         cls.tolookup_identities(layout, positions, sharedptrs, arrays)
@@ -562,7 +562,7 @@ class NumpyArrayType(ContentType):
         elif form.primitive == "uint8":
             arraytype = numba.types.Array(numba.uint8, 1, "A")
         elif form.primitive == "bool":
-            arraytype = numba.types.Array(numba.bool, 1, "A")
+            arraytype = numba.types.Array(numba.boolean, 1, "A")
         else:
             raise ValueError(
                 "unrecognized NumpyForm.primitive type: {0}".format(form.primitive)
@@ -1889,14 +1889,14 @@ class RecordArrayType(ContentType):
         positions.extend([None] * form.numfields)
         sharedptrs.extend([None] * form.numfields)
         if form.istuple:
-            for i, (n, content) in enumerate(form.contents.items()):
+            for i, (_, content) in enumerate(form.contents.items()):
                 positions[
                     pos + cls.CONTENTS + i
                 ] = ak._connect._numba.arrayview.tolookup(
                     content, positions, sharedptrs, arrays
                 )
         else:
-            for i, (n, content) in enumerate(form.contents.items()):
+            for i, (_, content) in enumerate(form.contents.items()):
                 positions[
                     pos + cls.CONTENTS + i
                 ] = ak._connect._numba.arrayview.tolookup(
@@ -1909,7 +1909,7 @@ class RecordArrayType(ContentType):
         contents = []
         if form.istuple:
             recordlookup = None
-            for n, x in form.contents.items():
+            for x in form.contents.values():
                 contents.append(ak._connect._numba.arrayview.tonumbatype(x))
         else:
             recordlookup = []
@@ -1941,10 +1941,18 @@ class RecordArrayType(ContentType):
         lookup.sharedptrs[pos] = lookup.sharedptrs_hold[pos].ptr()
         self.form_fill_identities(pos, layout, lookup)
 
-        for i, contenttype in enumerate(self.contenttypes):
-            contenttype.form_fill(
-                lookup.arrayptrs[pos + self.CONTENTS + i], layout.field(i), lookup
-            )
+        if self.recordlookup is None:
+            for i, contenttype in enumerate(self.contenttypes):
+                contenttype.form_fill(
+                    lookup.arrayptrs[pos + self.CONTENTS + i], layout.field(i), lookup
+                )
+        else:
+            for i, contenttype in enumerate(self.contenttypes):
+                contenttype.form_fill(
+                    lookup.arrayptrs[pos + self.CONTENTS + i],
+                    layout.field(self.recordlookup[i]),
+                    lookup,
+                )
 
     def fieldindex(self, key):
         out = -1
@@ -2595,7 +2603,7 @@ class VirtualArrayType(ContentType):
                 elif form.primitive == "uint8":
                     return numba.uint8
                 elif form.primitive == "bool":
-                    return numba.bool
+                    return numba.boolean
                 else:
                     raise ValueError(
                         "unrecognized NumpyForm.primitive type: {0}".format(
