@@ -5,7 +5,10 @@ from __future__ import absolute_import
 import re
 import sys
 import os
+import os.path
 import warnings
+import itertools
+import numbers
 
 try:
     from collections.abc import Mapping
@@ -29,6 +32,52 @@ if py27:
 else:
     unicode = None
 
+# matches include/awkward/common.h
+kMaxInt8 = 127  # 2**7  - 1
+kMaxUInt8 = 255  # 2**8  - 1
+kMaxInt32 = 2147483647  # 2**31 - 1
+kMaxUInt32 = 4294967295  # 2**32 - 1
+kMaxInt64 = 9223372036854775806  # 2**63 - 2: see below
+kSliceNone = kMaxInt64 + 1  # for Slice::none()
+kMaxLevels = 48
+
+
+def is_file_path(x):
+    try:
+        return os.path.isfile(x)
+    except ValueError:
+        return False
+
+
+def isint(x):
+    """
+    Returns True if and only if ``x`` is an integer (including NumPy, not
+    including bool).
+    """
+    return isinstance(
+        x, (int, numbers.Integral, np.integer, ak._v2._typetracer.Interval)
+    ) and not isinstance(x, (bool, np.bool_))
+
+
+def isnum(x):
+    """
+    Returns True if and only if ``x`` is a number (including NumPy, not
+    including bool).
+    """
+    return isinstance(x, (int, float, numbers.Real, np.number)) and not isinstance(
+        x, (bool, np.bool_)
+    )
+
+
+def isstr(x):
+    """
+    Returns True if and only if ``x`` is a string (including Python 2 unicode).
+    """
+    if py27:
+        return isinstance(x, (bytes, unicode))
+    else:
+        return isinstance(x, str)
+
 
 def exception_suffix(filename):
     line = ""
@@ -45,21 +94,40 @@ def exception_suffix(filename):
     )
 
 
-def deprecate(exception, version, date=None):
-    if ak.deprecations_as_errors:
-        raise exception
-    else:
-        if date is None:
-            date = ""
-        else:
-            date = " (target date: " + date + ")"
-        message = """In version {0}{1}, this will be an error.
-(Set ak.deprecations_as_errors = True to get a stack trace now.)
+# Enable warnings for the Awkward package
+warnings.filterwarnings("default", module="awkward.*")
 
-{2}: {3}""".format(
-            version, date, type(exception).__name__, str(exception)
-        )
-        warnings.warn(message, FutureWarning)
+
+def deprecate(
+    message,
+    version,
+    date=None,
+    will_be="an error",
+    category=DeprecationWarning,
+    stacklevel=2,
+):
+    if date is None:
+        date = ""
+    else:
+        date = " (target date: " + date + ")"
+    warning = """In version {0}{1}, this will be {2}.
+
+To raise these warnings as errors (and get stack traces to find out where they're called), run
+
+    import warnings
+    warnings.filterwarnings("error", module="awkward.*")
+
+after the first `import awkward` or use `@pytest.mark.filterwarnings("error:::awkward.*")` in pytest.
+
+Issue: {3}.""".format(
+        version, date, will_be, message
+    )
+    warnings.warn(warning, category, stacklevel=stacklevel + 1)
+
+
+# Sentinel object for catching pass-through values
+class MISSING(object):
+    pass
 
 
 virtualtypes = (ak.layout.VirtualArray,)
@@ -104,6 +172,30 @@ listtypes = (
 recordtypes = (ak.layout.RecordArray,)
 
 
+def regularize_path(path):
+    """
+    Converts pathlib Paths into plain string paths (for all versions of Python).
+    """
+    is_path = False
+
+    if isinstance(path, getattr(os, "PathLike", ())):
+        is_path = True
+        path = os.fspath(path)
+
+    elif hasattr(path, "__fspath__"):
+        is_path = True
+        path = path.__fspath__()
+
+    elif path.__class__.__module__ == "pathlib":
+        import pathlib
+
+        if isinstance(path, pathlib.Path):
+            is_path = True
+            path = str(path)
+
+    return is_path, path
+
+
 class Behavior(Mapping):
     def __init__(self, defaults, overrides):
         self.defaults = defaults
@@ -129,8 +221,8 @@ class Behavior(Mapping):
                 yield n, x
 
     def __iter__(self):
-        for n, x in self.items():
-            yield n
+        for x in self.keys():
+            yield x
 
     def __len__(self):
         return len(set(self.defaults) | set(self.overrides))
@@ -157,6 +249,19 @@ def arrayclass(layout, behavior):
     return ak.highlevel.Array
 
 
+def custom_cast(obj, behavior):
+    behavior = Behavior(ak.behavior, behavior)
+    for key, fcn in behavior.items():
+        if (
+            isinstance(key, tuple)
+            and len(key) == 2
+            and key[0] == "__cast__"
+            and isinstance(obj, key[1])
+        ):
+            return fcn
+    return None
+
+
 def custom_broadcast(layout, behavior):
     layout = ak.partition.first(layout)
     behavior = Behavior(ak.behavior, behavior)
@@ -243,6 +348,21 @@ def typestrs(behavior):
     return out
 
 
+def gettypestr(parameters, typestrs):
+    if parameters is not None:
+        record = parameters.get("__record__")
+        if record is not None:
+            typestr = typestrs.get(record)
+            if typestr is not None:
+                return typestr
+        array = parameters.get("__array__")
+        if array is not None:
+            typestr = typestrs.get(array)
+            if typestr is not None:
+                return typestr
+    return None
+
+
 def numba_record_typer(layouttype, behavior):
     behavior = Behavior(ak.behavior, behavior)
     rec = layouttype.parameters.get("__record__")
@@ -402,6 +522,17 @@ def wrap(content, behavior):
         return content
 
 
+def maybe_wrap(content, behavior, highlevel):
+    if highlevel:
+        return ak._util.wrap(content, behavior)
+    else:
+        return content
+
+
+def maybe_wrap_like(content, array, behavior, highlevel):
+    return maybe_wrap(content, behaviorof(array, behavior=behavior), highlevel)
+
+
 def extra(args, kwargs, defaults):
     out = []
     for i in range(len(defaults)):
@@ -460,7 +591,10 @@ def completely_flatten(array):
     elif isinstance(array, uniontypes):
         out = []
         for i in range(array.numcontents):
-            out.append(completely_flatten(array.project(i)))
+            tmp = completely_flatten(array.project(i))
+            assert isinstance(tmp, tuple)
+            for x in tmp:
+                out.append(x)
         return tuple(out)
 
     elif isinstance(array, optiontypes):
@@ -476,7 +610,15 @@ def completely_flatten(array):
         return tuple(out)
 
     elif isinstance(array, ak.layout.NumpyArray):
-        return (ak.nplike.of(array).asarray(array),)
+        if array.format.upper().startswith("M"):
+            return (
+                ak.nplike.of(array)
+                .asarray(array.view_int64)
+                .view(array.format)
+                .reshape(-1),
+            )
+        else:
+            return (ak.nplike.of(array).asarray(array).reshape(-1),)
 
     else:
         raise RuntimeError(
@@ -549,7 +691,10 @@ def broadcast_and_apply(  # noqa: C901
                 ):
                     return False
             elif isinstance(x, ak.layout.RegularArray):
-                my_offsets = nplike.arange(0, len(x.content), x.size)
+                if x.size == 0:
+                    my_offsets = nplike.empty(0, dtype=np.int64)
+                else:
+                    my_offsets = nplike.arange(0, len(x.content), x.size)
                 if offsets is None:
                     offsets = my_offsets
                 elif not nplike.array_equal(offsets, my_offsets):
@@ -650,10 +795,12 @@ def broadcast_and_apply(  # noqa: C901
 
         elif any(isinstance(x, uniontypes) for x in inputs):
             tagslist = []
+            numtags = []
             length = None
             for x in inputs:
                 if isinstance(x, uniontypes):
                     tagslist.append(nplike.asarray(x.tags))
+                    numtags.append(len(x.contents))
                     if length is None:
                         length = len(tagslist[-1])
                     elif length != len(tagslist[-1]):
@@ -666,19 +813,25 @@ def broadcast_and_apply(  # noqa: C901
                         )
 
             combos = nplike.stack(tagslist, axis=-1)
+
+            all_combos = nplike.array(
+                list(itertools.product(*[range(x) for x in numtags])),
+                dtype=[(str(i), combos.dtype) for i in range(len(tagslist))],
+            )
+
             combos = combos.view(
                 [(str(i), combos.dtype) for i in range(len(tagslist))]
             ).reshape(length)
 
             tags = nplike.empty(length, dtype=np.int8)
             index = nplike.empty(length, dtype=np.int64)
+            numoutputs = None
             outcontents = []
-            for tag, combo in enumerate(nplike.unique(combos)):
+            for tag, combo in enumerate(all_combos):
                 mask = combos == combo
                 tags[mask] = tag
                 index[mask] = nplike.arange(nplike.count_nonzero(mask))
                 nextinputs = []
-                numoutputs = None
                 i = 0
                 for x in inputs:
                     if isinstance(x, uniontypes):
@@ -694,9 +847,10 @@ def broadcast_and_apply(  # noqa: C901
                     assert numoutputs == len(outcontents[-1])
                 numoutputs = len(outcontents[-1])
 
+            assert numoutputs is not None
+
             tags = ak.layout.Index8(tags)
             index = ak.layout.Index64(index)
-
             return tuple(
                 ak.layout.UnionArray8_64(
                     tags, index, [x[i] for x in outcontents]
@@ -914,9 +1068,7 @@ def broadcast_and_apply(  # noqa: C901
         elif any(isinstance(x, recordtypes) for x in inputs):
             if not allow_records:
                 raise ValueError(
-                    "cannot broadcast: {0}".format(
-                        ", ".join(repr(type(x)) for x in inputs)
-                    )
+                    "cannot broadcast records in this type of operation"
                     + exception_suffix(__file__)
                 )
 
@@ -1074,7 +1226,7 @@ def recursively_apply(
     keep_parameters=True,
     numpy_to_regular=False,
 ):
-    def apply(layout, depth, user):
+    def transform(layout, depth, user):
         if numpy_to_regular and isinstance(layout, ak.layout.NumpyArray):
             layout = layout.toRegularArray()
 
@@ -1087,208 +1239,13 @@ def recursively_apply(
         custom = getfunction(layout, *args)
         if callable(custom):
             return custom()
-        else:
-            user = custom
-
-        # the rest of this is one switch statement
-        if isinstance(layout, ak.partition.PartitionedArray):
-            return ak.partition.IrregularlyPartitionedArray(
-                [apply(x, depth, user) for x in layout.partitions]
-            )
-
-        elif isinstance(layout, ak.layout.NumpyArray):
-            if keep_parameters:
-                return layout
-            else:
-                return ak.layout.NumpyArray(
-                    ak.nplike.of(layout).asarray(layout), layout.identities, None
-                )
-
-        elif isinstance(layout, ak.layout.EmptyArray):
-            if keep_parameters:
-                return layout
-            else:
-                return ak.layout.EmptyArray(layout.identities, None)
-
-        elif isinstance(layout, ak.layout.RegularArray):
-            return ak.layout.RegularArray(
-                apply(layout.content, depth + 1, user),
-                layout.size,
-                len(layout),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.ListArray32):
-            return ak.layout.ListArray32(
-                layout.starts,
-                layout.stops,
-                apply(layout.content, depth + 1, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.ListArrayU32):
-            return ak.layout.ListArrayU32(
-                layout.starts,
-                layout.stops,
-                apply(layout.content, depth + 1, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.ListArray64):
-            return ak.layout.ListArray64(
-                layout.starts,
-                layout.stops,
-                apply(layout.content, depth + 1, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.ListOffsetArray32):
-            return ak.layout.ListOffsetArray32(
-                layout.offsets,
-                apply(layout.content, depth + 1, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.ListOffsetArrayU32):
-            return ak.layout.ListOffsetArrayU32(
-                layout.offsets,
-                apply(layout.content, depth + 1, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.ListOffsetArray64):
-            return ak.layout.ListOffsetArray64(
-                layout.offsets,
-                apply(layout.content, depth + 1, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.IndexedArray32):
-            return ak.layout.IndexedArray32(
-                layout.index,
-                apply(layout.content, depth, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.IndexedArrayU32):
-            return ak.layout.IndexedArrayU32(
-                layout.index,
-                apply(layout.content, depth, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.IndexedArray64):
-            return ak.layout.IndexedArray64(
-                layout.index,
-                apply(layout.content, depth, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.IndexedOptionArray32):
-            return ak.layout.IndexedOptionArray32(
-                layout.index,
-                apply(layout.content, depth, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.IndexedOptionArray64):
-            return ak.layout.IndexedOptionArray64(
-                layout.index,
-                apply(layout.content, depth, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.ByteMaskedArray):
-            return ak.layout.ByteMaskedArray(
-                layout.mask,
-                apply(layout.content, depth, user),
-                layout.valid_when,
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.BitMaskedArray):
-            return ak.layout.BitMaskedArray(
-                layout.mask,
-                apply(layout.content, depth, user),
-                layout.valid_when,
-                len(layout),
-                layout.lsb_order,
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.UnmaskedArray):
-            return ak.layout.UnmaskedArray(
-                apply(layout.content, depth, user),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.RecordArray):
-            return ak.layout.RecordArray(
-                [apply(x, depth, user) for x in layout.contents],
-                layout.recordlookup,
-                len(layout),
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.Record):
-            return ak.layout.Record(
-                apply(layout.array, depth, user),
-                layout.at,
-            )
-
-        elif isinstance(layout, ak.layout.UnionArray8_32):
-            return ak.layout.UnionArray8_32(
-                layout.tags,
-                layout.index,
-                [apply(x, depth, user) for x in layout.contents],
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.UnionArray8_U32):
-            return ak.layout.UnionArray8_U32(
-                layout.tags,
-                layout.index,
-                [apply(x, depth, user) for x in layout.contents],
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.UnionArray8_64):
-            return ak.layout.UnionArray8_64(
-                layout.tags,
-                layout.index,
-                [apply(x, depth, user) for x in layout.contents],
-                layout.identities,
-                layout.parameters if keep_parameters else None,
-            )
-
-        elif isinstance(layout, ak.layout.VirtualArray):
-            return apply(layout.array, depth, user)
 
         else:
-            raise AssertionError(
-                "unrecognized Content type: {0}".format(type(layout))
-                + exception_suffix(__file__)
+            return transform_child_layouts(
+                transform, layout, depth, user=custom, keep_parameters=keep_parameters
             )
 
-    return apply(layout, 1, user)
+    return transform(layout, 1, user)
 
 
 def recursive_walk(layout, apply, args=(), depth=1, materialize=False):
@@ -1359,6 +1316,206 @@ def recursive_walk(layout, apply, args=(), depth=1, materialize=False):
         )
 
 
+def transform_child_layouts(transform, layout, depth, user=None, keep_parameters=True):
+    # the rest of this is one switch statement
+    if isinstance(layout, ak.partition.PartitionedArray):
+        return ak.partition.IrregularlyPartitionedArray(
+            [transform(x, depth, user) for x in layout.partitions]
+        )
+
+    elif isinstance(layout, ak.layout.NumpyArray):
+        if keep_parameters:
+            return layout
+        else:
+            return ak.layout.NumpyArray(
+                ak.nplike.of(layout).asarray(layout), layout.identities, None
+            )
+
+    elif isinstance(layout, ak.layout.EmptyArray):
+        if keep_parameters:
+            return layout
+        else:
+            return ak.layout.EmptyArray(layout.identities, None)
+
+    elif isinstance(layout, ak.layout.RegularArray):
+        return ak.layout.RegularArray(
+            transform(layout.content, depth + 1, user),
+            layout.size,
+            len(layout),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.ListArray32):
+        return ak.layout.ListArray32(
+            layout.starts,
+            layout.stops,
+            transform(layout.content, depth + 1, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.ListArrayU32):
+        return ak.layout.ListArrayU32(
+            layout.starts,
+            layout.stops,
+            transform(layout.content, depth + 1, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.ListArray64):
+        return ak.layout.ListArray64(
+            layout.starts,
+            layout.stops,
+            transform(layout.content, depth + 1, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.ListOffsetArray32):
+        return ak.layout.ListOffsetArray32(
+            layout.offsets,
+            transform(layout.content, depth + 1, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.ListOffsetArrayU32):
+        return ak.layout.ListOffsetArrayU32(
+            layout.offsets,
+            transform(layout.content, depth + 1, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.ListOffsetArray64):
+        return ak.layout.ListOffsetArray64(
+            layout.offsets,
+            transform(layout.content, depth + 1, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.IndexedArray32):
+        return ak.layout.IndexedArray32(
+            layout.index,
+            transform(layout.content, depth, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.IndexedArrayU32):
+        return ak.layout.IndexedArrayU32(
+            layout.index,
+            transform(layout.content, depth, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.IndexedArray64):
+        return ak.layout.IndexedArray64(
+            layout.index,
+            transform(layout.content, depth, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.IndexedOptionArray32):
+        return ak.layout.IndexedOptionArray32(
+            layout.index,
+            transform(layout.content, depth, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.IndexedOptionArray64):
+        return ak.layout.IndexedOptionArray64(
+            layout.index,
+            transform(layout.content, depth, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.ByteMaskedArray):
+        return ak.layout.ByteMaskedArray(
+            layout.mask,
+            transform(layout.content, depth, user),
+            layout.valid_when,
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.BitMaskedArray):
+        return ak.layout.BitMaskedArray(
+            layout.mask,
+            transform(layout.content, depth, user),
+            layout.valid_when,
+            len(layout),
+            layout.lsb_order,
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.UnmaskedArray):
+        return ak.layout.UnmaskedArray(
+            transform(layout.content, depth, user),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.RecordArray):
+        return ak.layout.RecordArray(
+            [transform(x, depth, user) for x in layout.contents],
+            layout.recordlookup,
+            len(layout),
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.Record):
+        return ak.layout.Record(
+            transform(layout.array, depth, user),
+            layout.at,
+        )
+
+    elif isinstance(layout, ak.layout.UnionArray8_32):
+        return ak.layout.UnionArray8_32(
+            layout.tags,
+            layout.index,
+            [transform(x, depth, user) for x in layout.contents],
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.UnionArray8_U32):
+        return ak.layout.UnionArray8_U32(
+            layout.tags,
+            layout.index,
+            [transform(x, depth, user) for x in layout.contents],
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.UnionArray8_64):
+        return ak.layout.UnionArray8_64(
+            layout.tags,
+            layout.index,
+            [transform(x, depth, user) for x in layout.contents],
+            layout.identities,
+            layout.parameters if keep_parameters else None,
+        )
+
+    elif isinstance(layout, ak.layout.VirtualArray):
+        return transform(layout.array, depth, user)
+
+    else:
+        raise AssertionError(
+            "unrecognized Content type: {0}".format(type(layout))
+            + exception_suffix(__file__)
+        )
+
+
 def find_caches(layout):
     # Both of the implementations below find referentially unique mutablemappings,
     # but the PartitionedArray case is optimized for many unique values (with a set)
@@ -1455,6 +1612,8 @@ def minimally_touching_string(limit_length, layout, behavior):
                         key = ""
                     sp = ", "
                 yield "}"
+            elif isinstance(x, (np.datetime64, np.timedelta64)):
+                yield space + str(x)
             elif isinstance(x, (float, np.floating)):
                 yield space + "{0:.3g}".format(x)
             else:
@@ -1520,6 +1679,8 @@ def minimally_touching_string(limit_length, layout, behavior):
                     if i != 0:
                         yield ", "
                 yield "{"
+            elif isinstance(x, (np.datetime64, np.timedelta64)):
+                yield str(x) + space
             elif isinstance(x, (float, np.floating)):
                 yield "{0:.3g}".format(x) + space
             else:
@@ -1667,7 +1828,9 @@ def union_to_record(unionarray, anonymous):
             contents.append(union_to_record(layout, anonymous))
         elif isinstance(layout, optiontypes):
             contents.append(
-                ak.operations.structure.fill_none(layout, np.nan, highlevel=False)
+                ak.operations.structure.fill_none(
+                    layout, np.nan, axis=0, highlevel=False
+                )
             )
         else:
             contents.append(layout)
