@@ -16,7 +16,7 @@ def count(array, axis=None, keepdims=False, mask_identity=False):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -107,7 +107,7 @@ def count_nonzero(array, axis=None, keepdims=False, mask_identity=False):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -164,7 +164,7 @@ def sum(array, axis=None, keepdims=False, mask_identity=False):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -357,7 +357,7 @@ def prod(array, axis=None, keepdims=False, mask_identity=False):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -406,7 +406,7 @@ def any(array, axis=None, keepdims=False, mask_identity=False):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -457,7 +457,7 @@ def all(array, axis=None, keepdims=False, mask_identity=False):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -508,7 +508,7 @@ def min(array, axis=None, keepdims=False, initial=None, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -569,7 +569,7 @@ def max(array, axis=None, keepdims=False, initial=None, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -620,6 +620,74 @@ def max(array, axis=None, keepdims=False, initial=None, mask_identity=True):
         )
 
 
+@ak._connect._numpy.implements("ptp")
+def ptp(arr, axis=None, keepdims=False, mask_identity=True):
+    """
+    Args:
+        array: Data from which to find the range of values.
+        axis (None or int): If None, combine all values from the array into
+            a single scalar result; if an int, group by that axis: `0` is the
+            outermost, `1` is the first level of nested lists, etc., and
+            negative `axis` counts from the innermost: `-1` is the innermost,
+            `-2` is the next level up, etc.
+        keepdims (bool): If False, this reducer decreases the number of
+            dimensions by 1; if True, the reduced values are wrapped in a new
+            length-1 dimension so that the result of this operation may be
+            broadcasted with the original array.
+        mask_identity (bool): If True, reducing over empty lists results in
+            None (an option type); otherwise, reducing over empty lists
+            results in the operation's identity of 0.
+
+    Returns the range of values in each group of elements from `array` (many
+    types supported, including all Awkward Arrays and Records). The range of
+    an empty list is None, unless `mask_identity=False`, in which case it is 0.
+    This operation is the same as NumPy's
+    [ptp](https://docs.scipy.org/doc/numpy/reference/generated/numpy.ptp.html)
+    if all lists at a given dimension have the same length and no None values,
+    but it generalizes to cases where they do not.
+
+    For example, with an `array` like
+
+        ak.Array([[0, 1, 2, 3],
+                  [          ],
+                  [4, 5      ]])
+
+    The range of the innermost lists is
+
+        >>> ak.ptp(array, axis=-1)
+        <Array [3, None, 1] type='3 * ?float64'>
+
+    because there are three lists, the first has a range of `3`, the second is
+    `None` because the list is empty, and the third has a range of `1`. Similarly,
+
+        >>> ak.ptp(array, axis=-1, mask_identity=False)
+        <Array [3, 0, 1] type='3 * float64'>
+
+    The second value is `0` because the list is empty.
+
+    See #ak.sum for a more complete description of nested list and missing
+    value (None) handling in reducers.
+    """
+    if axis is None:
+        out = ak.max(arr) - ak.min(arr)
+        if not mask_identity and out is None:
+            out = 0
+
+    else:
+        maxi = ak.max(arr, axis=axis, mask_identity=True, keepdims=True)
+        mini = ak.min(arr, axis=axis, mask_identity=True, keepdims=True)
+        out = maxi - mini
+
+        if not mask_identity:
+            out = ak.fill_none(out, 0, axis=-1)
+
+        if not keepdims:
+            posaxis = out.layout.axis_wrap_if_negative(axis)
+            out = out[(slice(None, None),) * posaxis + (0,)]
+
+    return out
+
+
 @ak._connect._numpy.implements("argmin")
 def argmin(array, axis=None, keepdims=False, mask_identity=True):
     """
@@ -630,7 +698,7 @@ def argmin(array, axis=None, keepdims=False, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -700,7 +768,7 @@ def argmax(array, axis=None, keepdims=False, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this reducer descreases the number of
+        keepdims (bool): If False, this reducer decreases the number of
             dimensions by 1; if True, the reduced values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -779,7 +847,7 @@ def moment(x, n, weight=None, axis=None, keepdims=False, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this function descreases the number of
+        keepdims (bool): If False, this function decreases the number of
             dimensions by 1; if True, the output values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -842,7 +910,7 @@ def mean(x, weight=None, axis=None, keepdims=False, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this function descreases the number of
+        keepdims (bool): If False, this function decreases the number of
             dimensions by 1; if True, the output values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -924,7 +992,7 @@ def var(x, weight=None, ddof=0, axis=None, keepdims=False, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this function descreases the number of
+        keepdims (bool): If False, this function decreases the number of
             dimensions by 1; if True, the output values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -1006,7 +1074,7 @@ def std(x, weight=None, ddof=0, axis=None, keepdims=False, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this function descreases the number of
+        keepdims (bool): If False, this function decreases the number of
             dimensions by 1; if True, the output values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -1059,7 +1127,7 @@ def covar(x, y, weight=None, axis=None, keepdims=False, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this function descreases the number of
+        keepdims (bool): If False, this function decreases the number of
             dimensions by 1; if True, the output values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -1128,7 +1196,7 @@ def corr(x, y, weight=None, axis=None, keepdims=False, mask_identity=True):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this function descreases the number of
+        keepdims (bool): If False, this function decreases the number of
             dimensions by 1; if True, the output values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -1210,7 +1278,7 @@ def linear_fit(x, y, weight=None, axis=None, keepdims=False, mask_identity=True)
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this function descreases the number of
+        keepdims (bool): If False, this function decreases the number of
             dimensions by 1; if True, the output values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -1404,7 +1472,7 @@ def softmax(x, axis=None, keepdims=False, mask_identity=False):
             outermost, `1` is the first level of nested lists, etc., and
             negative `axis` counts from the innermost: `-1` is the innermost,
             `-2` is the next level up, etc.
-        keepdims (bool): If False, this function descreases the number of
+        keepdims (bool): If False, this function decreases the number of
             dimensions by 1; if True, the output values are wrapped in a new
             length-1 dimension so that the result of this operation may be
             broadcasted with the original array.
@@ -1436,7 +1504,9 @@ def softmax(x, axis=None, keepdims=False, mask_identity=False):
 
 
 __all__ = [
-    x
-    for x in list(globals())
-    if not x.startswith("_") and x not in ("absolute_import", "np", "awkward")
+    x for x in list(globals()) if not x.startswith("_") and x not in ("ak", "np")
 ]
+
+
+def __dir__():
+    return __all__
