@@ -256,7 +256,16 @@ namespace awkward {
         json["class"].IsString()) {
 
       bool h = false;
-      if (json.HasMember("has_identities")) {
+      if (json.HasMember("has_identifier")) {
+        if (json["has_identifier"].IsBool()) {
+          h = json["has_identifier"].GetBool();
+        }
+        else {
+          throw std::invalid_argument(
+            std::string("'has_identifier' must be boolean") + FILENAME(__LINE__));
+        }
+      }
+      else if (json.HasMember("has_identities")) {
         if (json["has_identities"].IsBool()) {
           h = json["has_identities"].GetBool();
         }
@@ -361,6 +370,7 @@ namespace awkward {
         return std::make_shared<RecordForm>(h, p, f, recordlookup, contents);
       }
 
+      isgen = is64 = isU32 = is32 = false;
       if ((isgen = (cls == std::string("ListOffsetArray")))  ||
           (is64  = (cls == std::string("ListOffsetArray64")))  ||
           (isU32 = (cls == std::string("ListOffsetArrayU32")))  ||
@@ -391,6 +401,7 @@ namespace awkward {
         return std::make_shared<ListOffsetForm>(h, p, f, offsets, content);
       }
 
+      isgen = is64 = isU32 = is32 = false;
       if ((isgen = (cls == std::string("ListArray")))  ||
           (is64  = (cls == std::string("ListArray64")))  ||
           (isU32 = (cls == std::string("ListArrayU32")))  ||
@@ -454,6 +465,7 @@ namespace awkward {
         return std::make_shared<RegularForm>(h, p, f, content, size);
       }
 
+      isgen = is64 = is32 = false;
       if ((isgen = (cls == std::string("IndexedOptionArray")))  ||
           (is64  = (cls == std::string("IndexedOptionArray64")))  ||
           (is32  = (cls == std::string("IndexedOptionArray32")))) {
@@ -482,6 +494,7 @@ namespace awkward {
         return std::make_shared<IndexedOptionForm>(h, p, f, index, content);
       }
 
+      isgen = is64 = isU32 = is32 = false;
       if ((isgen = (cls == std::string("IndexedArray")))  ||
           (is64  = (cls == std::string("IndexedArray64")))  ||
           (isU32 = (cls == std::string("IndexedArrayU32")))  ||
@@ -595,6 +608,7 @@ namespace awkward {
         return std::make_shared<UnmaskedForm>(h, p, f, content);
       }
 
+      isgen = is64 = isU32 = is32 = false;
       if ((isgen = (cls == std::string("UnionArray")))  ||
           (is64  = (cls == std::string("UnionArray8_64")))  ||
           (isU32 = (cls == std::string("UnionArray8_U32")))  ||
@@ -1084,25 +1098,29 @@ namespace awkward {
 
     Index64 starts(1);
     starts.setitem_at_nowrap(0, 0);
+
+    Index64 shifts(0);
+
     Index64 parents(length());
     struct Error err = kernel::content_reduce_zeroparents_64(
       kernel::lib::cpu,   // DERIVE
       parents.data(),
       length());
     util::handle_error(err, classname(), identities_.get());
+
     ContentPtr out = argsort_next(negaxis,
                                   starts,
+                                  shifts,
                                   parents,
                                   1,
                                   ascending,
-                                  stable,
-                                  true);
+                                  stable);
 
      if (out.get()->length() == 0) {
        return out.get()->getitem_nothing();
      }
      else {
-       return out.get()->getitem_at_nowrap(0);
+       return out;
      }
   }
 
@@ -1161,14 +1179,13 @@ namespace awkward {
                                parents,
                                1,
                                ascending,
-                               stable,
-                               true);
+                               stable);
 
     if (out.get()->length() == 0) {
       return out.get()->getitem_nothing();
     }
     else {
-      return out.get()->getitem_at_nowrap(0);
+      return out;
     }
   }
 
@@ -1431,10 +1448,6 @@ namespace awkward {
              dynamic_cast<SliceJagged64*>(head.get())) {
       return getitem_next(*jagged, tail, advanced);
     }
-    else if (SliceVarNewAxis* varnewaxis =
-             dynamic_cast<SliceVarNewAxis*>(head.get())) {
-      return getitem_next(*varnewaxis, tail, advanced);
-    }
     else {
       throw std::runtime_error(
         std::string("unrecognized slice type") + FILENAME(__LINE__));
@@ -1458,10 +1471,6 @@ namespace awkward {
              dynamic_cast<SliceJagged64*>(slicecontent.get())) {
       return getitem_next_jagged(slicestarts, slicestops, *jagged, tail);
     }
-    else if (SliceVarNewAxis* varnewaxis =
-             dynamic_cast<SliceVarNewAxis*>(slicecontent.get())) {
-      return getitem_next_jagged(slicestarts, slicestops, *varnewaxis, tail);
-    }
     else {
       throw std::runtime_error(
         std::string("unexpected slice type for getitem_next_jagged")
@@ -1745,6 +1754,26 @@ namespace awkward {
                 + std::string("): __array__ = \"string\" must directly contain "
                               "a node with __array__ = \"char\""));
       }
+      if (const NumpyArray* raw = dynamic_cast<const NumpyArray*>(content.get())) {
+        if (raw->dtype() != util::dtype::uint8) {
+          return (std::string("at ") + path + std::string(" (") + raw->classname()
+                  + std::string("): __array__ = \"char\" requires dtype == uint8"));
+        }
+        if (raw->shape().size() != 1) {
+          return (std::string("at ") + path + std::string(" (") + raw->classname()
+                  + std::string("): __array__ = \"char\" must be one-dimensional"));
+        }
+        /// I don't think this is strictly required.
+        // if (raw->stride() != raw->itemsize()) {
+        //   return (std::string("at ") + path + std::string(" (") + classname()
+        //           + std::string("): __array__ = \"char\" must be contiguous"));
+        // }
+      }
+      else {
+        return (std::string("at ") + path + std::string(" (") + raw->classname()
+                + std::string("): __array__ = \"char\" only allowed for NumpyArray"));
+      }
+      return std::string("");
     }
 
     if (parameter_equals("__array__", "\"bytestring\"")) {
@@ -1780,69 +1809,67 @@ namespace awkward {
                 + std::string("): __array__ = \"bytestring\" must directly contain "
                               "a node with __array__ = \"byte\""));
       }
-    }
-
-    if (parameter_equals("__array__", "\"char\"")) {
-      if (const NumpyArray* raw = dynamic_cast<const NumpyArray*>(this)) {
+      if (const NumpyArray* raw = dynamic_cast<const NumpyArray*>(content.get())) {
         if (raw->dtype() != util::dtype::uint8) {
-          return (std::string("at ") + path + std::string(" (") + classname()
-                  + std::string("): __array__ = \"char\" requires dtype == uint8"));
-        }
-        if (raw->shape().size() != 1) {
-          return (std::string("at ") + path + std::string(" (") + classname()
-                  + std::string("): __array__ = \"char\" must be one-dimensional"));
-        }
-        /// I don't think this is strictly required.
-        // if (raw->stride() != raw->itemsize()) {
-        //   return (std::string("at ") + path + std::string(" (") + classname()
-        //           + std::string("): __array__ = \"char\" must be contiguous"));
-        // }
-      }
-      else {
-        return (std::string("at ") + path + std::string(" (") + classname()
-                + std::string("): __array__ = \"char\" only allowed for NumpyArray"));
-      }
-    }
-
-    if (parameter_equals("__array__", "\"byte\"")) {
-      if (const NumpyArray* raw = dynamic_cast<const NumpyArray*>(this)) {
-        if (raw->dtype() != util::dtype::uint8) {
-          return (std::string("at ") + path + std::string(" (") + classname()
+          return (std::string("at ") + path + std::string(" (") + raw->classname()
                   + std::string("): __array__ = \"byte\" requires dtype == uint8"));
         }
         if (raw->shape().size() != 1) {
-          return (std::string("at ") + path + std::string(" (") + classname()
+          return (std::string("at ") + path + std::string(" (") + raw->classname()
                   + std::string("): __array__ = \"byte\" must be one-dimensional"));
         }
         /// I don't think this is strictly required.
         // if (raw->stride() != raw->itemsize()) {
-        //   return (std::string("at ") + path + std::string(" (") + classname()
+        //   return (std::string("at ") + path + std::string(" (") + raw->classname()
         //           + std::string("): __array__ = \"byte\" must be contiguous"));
         // }
       }
       else {
-        return (std::string("at ") + path + std::string(" (") + classname()
+        return (std::string("at ") + path + std::string(" (") + raw->classname()
                 + std::string("): __array__ = \"byte\" only allowed for NumpyArray"));
       }
+      return std::string("");
+    }
+
+    if (parameter_equals("__array__", "\"char\"")) {
+      return (std::string("at ") + path + std::string(" (") + classname()
+              + std::string("): __array__ = \"char\" must be directly inside "
+                            "__array__ = \"string\""));
+    }
+
+    if (parameter_equals("__array__", "\"byte\"")) {
+      return (std::string("at ") + path + std::string(" (") + classname()
+              + std::string("): __array__ = \"byte\" must be directly inside "
+                            "__array__ = \"bytestring\""));
     }
 
     if (parameter_equals("__array__", "\"categorical\"")) {
-      if (dynamic_cast<const IndexedArray32*>(this)  ||
-          dynamic_cast<const IndexedArrayU32*>(this)  ||
-          dynamic_cast<const IndexedArray64*>(this)  ||
-          dynamic_cast<const IndexedOptionArray32*>(this)  ||
-          dynamic_cast<const IndexedOptionArray64*>(this)) {
-        if (!is_unique()) {
-          return (std::string("at ") + path + std::string(" (") + classname()
-                  + std::string("): __array__ = \"categorical\" requires contents "
-                                "to be unique"));
-        }
+      ContentPtr content(nullptr);
+      if (const IndexedArray32* raw = dynamic_cast<const IndexedArray32*>(this)) {
+        content = raw->content();
+      }
+      else if (const IndexedArrayU32* raw = dynamic_cast<const IndexedArrayU32*>(this)) {
+        content = raw->content();
+      }
+      else if (const IndexedArray64* raw = dynamic_cast<const IndexedArray64*>(this)) {
+        content = raw->content();
+      }
+      else if (const IndexedOptionArray32* raw = dynamic_cast<const IndexedOptionArray32*>(this)) {
+        content = raw->content();
+      }
+      else if (const IndexedOptionArray64* raw = dynamic_cast<const IndexedOptionArray64*>(this)) {
+        content = raw->content();
       }
       else {
         return (std::string("at ") + path + std::string(" (") + classname()
                 + std::string("): __array__ = \"categorical\" only allowed for "
                               "IndexedArray and IndexedOptionArray"));
       }
+      if (!content.get()->is_unique()) {
+        return (std::string("at ") + path + std::string(" (") + classname()
+            + std::string("): __array__ = \"categorical\" requires contents "
+                            "to be unique"));
+      }
     }
 
     // No constraints on __record__ YET.
