@@ -805,6 +805,17 @@ namespace awkward {
               + std::string("): ") + std::string("len(content) < length")
               + FILENAME(__LINE__));
     }
+    else if (dynamic_cast<BitMaskedArray*>(content_.get())  ||
+             dynamic_cast<ByteMaskedArray*>(content_.get())  ||
+             dynamic_cast<IndexedArray32*>(content_.get())  ||
+             dynamic_cast<IndexedArrayU32*>(content_.get())  ||
+             dynamic_cast<IndexedArray64*>(content_.get())  ||
+             dynamic_cast<IndexedOptionArray32*>(content_.get())  ||
+             dynamic_cast<IndexedOptionArray64*>(content_.get())  ||
+             dynamic_cast<UnmaskedArray*>(content_.get())) {
+      return classname() + " contains " + content_.get()->classname() +
+             ", the operation that made it might have forgotten to call 'simplify_optiontype()'";
+    }
     else {
       return content_.get()->validityerror(path + std::string(".content"));
     }
@@ -985,32 +996,30 @@ namespace awkward {
                             const Index64& parents,
                             int64_t outlength,
                             bool ascending,
-                            bool stable,
-                            bool keepdims) const {
+                            bool stable) const {
     return toByteMaskedArray().get()->sort_next(negaxis,
                                                 starts,
                                                 parents,
                                                 outlength,
                                                 ascending,
-                                                stable,
-                                                keepdims);
+                                                stable);
   }
 
   const ContentPtr
   BitMaskedArray::argsort_next(int64_t negaxis,
                                const Index64& starts,
+                               const Index64& shifts,
                                const Index64& parents,
                                int64_t outlength,
                                bool ascending,
-                               bool stable,
-                               bool keepdims) const {
+                               bool stable) const {
     return toByteMaskedArray().get()->argsort_next(negaxis,
                                                    starts,
+                                                   shifts,
                                                    parents,
                                                    outlength,
                                                    ascending,
-                                                   stable,
-                                                   keepdims);
+                                                   stable);
   }
 
   const ContentPtr
@@ -1082,30 +1091,6 @@ namespace awkward {
                                                           tail);
   }
 
-  const ContentPtr
-  BitMaskedArray::getitem_next_jagged(const Index64& slicestarts,
-                                      const Index64& slicestops,
-                                      const SliceVarNewAxis& slicecontent,
-                                      const Slice& tail) const {
-    return toByteMaskedArray().get()->getitem_next_jagged(slicestarts,
-                                                          slicestops,
-                                                          slicecontent,
-                                                          tail);
-  }
-
-  const ContentPtr
-  BitMaskedArray::getitem_next(const SliceVarNewAxis& varnewaxis,
-                               const Slice& tail,
-                               const Index64& advanced) const {
-    SliceJagged64 jagged = content_.get()->varaxis_to_jagged(varnewaxis);
-    return getitem_next(jagged, tail, advanced);
-  }
-
-  const SliceJagged64
-  BitMaskedArray::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    return content_.get()->varaxis_to_jagged(varnewaxis);
-  }
-
   const ContentPtr
   BitMaskedArray::copy_to(kernel::lib ptr_lib) const {
     IndexU8 mask = mask_.copy_to(ptr_lib);
