@@ -745,10 +745,6 @@ namespace awkward {
              dynamic_cast<SliceMissing64*>(head.get())) {
       return Content::getitem_next(*missing, tail, advanced);
     }
-    else if (SliceVarNewAxis* varnewaxis =
-             dynamic_cast<SliceVarNewAxis*>(head.get())) {
-      return ByteMaskedArray::getitem_next(*varnewaxis, tail, advanced);
-    }
     else {
       throw std::runtime_error(
         std::string("unrecognized slice type") + FILENAME(__LINE__));
@@ -836,6 +832,17 @@ namespace awkward {
               + std::string("): ") + std::string("len(content) < len(mask)")
               + FILENAME(__LINE__));
     }
+    else if (dynamic_cast<BitMaskedArray*>(content_.get())  ||
+             dynamic_cast<ByteMaskedArray*>(content_.get())  ||
+             dynamic_cast<IndexedArray32*>(content_.get())  ||
+             dynamic_cast<IndexedArrayU32*>(content_.get())  ||
+             dynamic_cast<IndexedArray64*>(content_.get())  ||
+             dynamic_cast<IndexedOptionArray32*>(content_.get())  ||
+             dynamic_cast<IndexedOptionArray64*>(content_.get())  ||
+             dynamic_cast<UnmaskedArray*>(content_.get())) {
+      return classname() + " contains " + content_.get()->classname() +
+             ", the operation that made it might have forgotten to call 'simplify_optiontype()'";
+    }
     else {
       return content_.get()->validityerror(path + std::string(".content"));
     }
@@ -1188,10 +1195,10 @@ namespace awkward {
           raw->identities(),
           raw->parameters(),
           outoffsets,
-          std::make_shared<IndexedOptionArray64>(Identities::none(),
-                                                 util::Parameters(),
-                                                 outindex,
-                                                 raw->content()));
+          IndexedOptionArray64(Identities::none(),
+                               util::Parameters(),
+                               outindex,
+                               raw->content()).simplify_optiontype());
       }
       else {
         throw std::runtime_error(
@@ -1267,172 +1274,30 @@ namespace awkward {
                              const Index64& parents,
                              int64_t outlength,
                              bool ascending,
-                             bool stable,
-                             bool keepdims) const {
-    if (length() == 0 ) {
-      return shallow_copy();
-    }
-
-    int64_t numnull;
-    struct Error err1 = kernel::ByteMaskedArray_numnull(
-      kernel::lib::cpu,   // DERIVE
-      &numnull,
-      mask_.data(),
-      length(),
-      valid_when_);
-    util::handle_error(err1, classname(), identities_.get());
-
-    Index64 nextparents(length() - numnull);
-    Index64 nextcarry(length() - numnull);
-    Index64 outindex(length());
-    struct Error err2 = kernel::ByteMaskedArray_reduce_next_64(
-      kernel::lib::cpu,   // DERIVE
-      nextcarry.data(),
-      nextparents.data(),
-      outindex.data(),
-      mask_.data(),
-      parents.data(),
-      length(),
-      valid_when_);
-    util::handle_error(err2, classname(), identities_.get());
-
-    ContentPtr next = content_.get()->carry(nextcarry, false);
-    ContentPtr out = next.get()->sort_next(negaxis,
-                                           starts,
-                                           nextparents,
-                                           outlength,
-                                           ascending,
-                                           stable,
-                                           keepdims);
-
-    std::pair<bool, int64_t> branchdepth = branch_depth();
-    if (!branchdepth.first  &&  negaxis == branchdepth.second) {
-      return out;
-    }
-    else {
-      if (RegularArray* raw =
-          dynamic_cast<RegularArray*>(out.get())) {
-        out = raw->toListOffsetArray64(true);
-      }
-      if (ListOffsetArray64* raw =
-          dynamic_cast<ListOffsetArray64*>(out.get())) {
-        Index64 outoffsets(starts.length() + 1);
-        if (starts.length() > 0  &&  starts.getitem_at_nowrap(0) != 0) {
-          throw std::runtime_error(
-            std::string("sort_next with unbranching depth > negaxis expects "
-                        "a ListOffsetArray64 whose offsets start at zero")
-            + FILENAME(__LINE__));
-        }
-        struct Error err3 = kernel::IndexedArray_reduce_next_fix_offsets_64(
-          kernel::lib::cpu,   // DERIVE
-          outoffsets.data(),
-          starts.data(),
-          starts.length(),
-          outindex.length());
-        util::handle_error(err3, classname(), identities_.get());
-
-        return std::make_shared<ListOffsetArray64>(
-          raw->identities(),
-          raw->parameters(),
-          outoffsets,
-          std::make_shared<IndexedOptionArray64>(Identities::none(),
-                                                 parameters_,
-                                                 outindex,
-                                                 raw->content()));
-      }
-      else {
-        throw std::runtime_error(
-          std::string("sort_next with unbranching depth > negaxis is only "
-                      "expected to return RegularArray or ListOffsetArray64; "
-                      "instead, it returned ")
-          + out.get()->classname() + FILENAME(__LINE__));
-      }
-    }
+                             bool stable) const {
+    return toIndexedOptionArray64().get()->sort_next(negaxis,
+                                                     starts,
+                                                     parents,
+                                                     outlength,
+                                                     ascending,
+                                                     stable);
   }
 
   const ContentPtr
   ByteMaskedArray::argsort_next(int64_t negaxis,
                                 const Index64& starts,
+                                const Index64& shifts,
                                 const Index64& parents,
                                 int64_t outlength,
                                 bool ascending,
-                                bool stable,
-                                bool keepdims) const {
-    int64_t numnull;
-    struct Error err1 = kernel::ByteMaskedArray_numnull(
-      kernel::lib::cpu,   // DERIVE
-      &numnull,
-      mask_.data(),
-      length(),
-      valid_when_);
-    util::handle_error(err1, classname(), identities_.get());
-
-    Index64 nextparents(length() - numnull);
-    Index64 nextcarry(length() - numnull);
-    Index64 outindex(length());
-    struct Error err2 = kernel::ByteMaskedArray_reduce_next_64(
-      kernel::lib::cpu,   // DERIVE
-      nextcarry.data(),
-      nextparents.data(),
-      outindex.data(),
-      mask_.data(),
-      parents.data(),
-      length(),
-      valid_when_);
-    util::handle_error(err2, classname(), identities_.get());
-
-    ContentPtr next = content_.get()->carry(nextcarry, false);
-    ContentPtr out = next.get()->argsort_next(negaxis,
-                                              starts,
-                                              nextparents,
-                                              outlength,
-                                              ascending,
-                                              stable,
-                                              keepdims);
-
-    std::pair<bool, int64_t> branchdepth = branch_depth();
-    if (!branchdepth.first  &&  negaxis == branchdepth.second) {
-      return out;
-    }
-    else {
-      if (RegularArray* raw =
-          dynamic_cast<RegularArray*>(out.get())) {
-        out = raw->toListOffsetArray64(true);
-      }
-      if (ListOffsetArray64* raw =
-          dynamic_cast<ListOffsetArray64*>(out.get())) {
-        Index64 outoffsets(starts.length() + 1);
-        if (starts.length() > 0  &&  starts.getitem_at_nowrap(0) != 0) {
-          throw std::runtime_error(
-            std::string("argsort_next with unbranching depth > negaxis expects "
-                        "a ListOffsetArray64 whose offsets start at zero")
-            + FILENAME(__LINE__));
-        }
-        struct Error err3 = kernel::IndexedArray_reduce_next_fix_offsets_64(
-          kernel::lib::cpu,   // DERIVE
-          outoffsets.data(),
-          starts.data(),
-          starts.length(),
-          outindex.length());
-        util::handle_error(err3, classname(), identities_.get());
-
-        return std::make_shared<ListOffsetArray64>(
-          raw->identities(),
-          raw->parameters(),
-          outoffsets,
-          std::make_shared<IndexedOptionArray64>(Identities::none(),
-                                                 util::Parameters(),
-                                                 outindex,
-                                                 raw->content()));
-      }
-      else {
-        throw std::runtime_error(
-          std::string("argsort_next with unbranching depth > negaxis is only "
-                      "expected to return RegularArray or ListOffsetArray64; "
-                      "instead, it returned ")
-          + out.get()->classname() + FILENAME(__LINE__));
-      }
-    }
+                                bool stable) const {
+    return toIndexedOptionArray64().get()->argsort_next(negaxis,
+                                                        starts,
+                                                        shifts,
+                                                        parents,
+                                                        outlength,
+                                                        ascending,
+                                                        stable);
   }
 
   const ContentPtr
@@ -1504,30 +1369,6 @@ namespace awkward {
                                                       tail);
   }
 
-  const ContentPtr
-  ByteMaskedArray::getitem_next_jagged(const Index64& slicestarts,
-                                       const Index64& slicestops,
-                                       const SliceVarNewAxis& slicecontent,
-                                       const Slice& tail) const {
-    return getitem_next_jagged_generic<SliceVarNewAxis>(slicestarts,
-                                                        slicestops,
-                                                        slicecontent,
-                                                        tail);
-  }
-
-  const ContentPtr
-  ByteMaskedArray::getitem_next(const SliceVarNewAxis& varnewaxis,
-                                const Slice& tail,
-                                const Index64& advanced) const {
-    SliceJagged64 jagged = content_.get()->varaxis_to_jagged(varnewaxis);
-    return getitem_next(jagged, tail, advanced);
-  }
-
-  const SliceJagged64
-  ByteMaskedArray::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    return content_.get()->varaxis_to_jagged(varnewaxis);
-  }
-
   const ContentPtr
   ByteMaskedArray::copy_to(kernel::lib ptr_lib) const {
     Index8 mask = mask_.copy_to(ptr_lib);
@@ -1560,8 +1401,18 @@ namespace awkward {
 
   template <typename S>
   const ContentPtr ByteMaskedArray::getitem_next_jagged_generic(
-      const Index64& slicestarts, const Index64& slicestops,
-      const S& slicecontent, const Slice& tail) const {
+      const Index64& slicestarts,
+      const Index64& slicestops,
+      const S& slicecontent,
+      const Slice& tail) const {
+    if (slicestarts.length() != length()) {
+      throw std::invalid_argument(
+        std::string("cannot fit jagged slice with length ")
+        + std::to_string(slicestarts.length()) + std::string(" into ")
+        + classname() + std::string(" of size ") + std::to_string(length())
+        + FILENAME(__LINE__));
+    }
+
     int64_t numnull;
     std::pair<Index64, Index64> pair = nextcarry_outindex(numnull);
     Index64 nextcarry = pair.first;
