@@ -603,35 +603,34 @@ namespace awkward {
                         const Index64& parents,
                         int64_t outlength,
                         bool ascending,
-                        bool stable,
-                        bool keepdims) const {
+                        bool stable) const {
     ContentPtr asnumpy = toNumpyArray("d", 8, util::dtype::float64);
     return asnumpy.get()->sort_next(negaxis,
                                     starts,
                                     parents,
                                     outlength,
                                     ascending,
-                                    stable,
-                                    keepdims);
+                                    stable);
   }
 
   const ContentPtr
   EmptyArray::argsort_next(int64_t negaxis,
                            const Index64& starts,
+                           const Index64& shifts,
                            const Index64& parents,
                            int64_t outlength,
                            bool ascending,
-                           bool stable,
-                           bool keepdims) const {
-    ContentPtr asnumpy = toNumpyArray("d", 8, util::dtype::float64);
-    ContentPtr out = asnumpy.get()->argsort_next(negaxis,
-                                                 starts,
-                                                 parents,
-                                                 outlength,
-                                                 ascending,
-                                                 stable,
-                                                 keepdims);
-    return out;
+                           bool stable) const {
+    ContentPtr asnumpy = toNumpyArray(util::dtype_to_format(util::dtype::int64),
+                                      util::dtype_to_itemsize(util::dtype::int64),
+                                      util::dtype::int64);
+    return asnumpy.get()->argsort_next(negaxis,
+                                       starts,
+                                       shifts,
+                                       parents,
+                                       outlength,
+                                       ascending,
+                                       stable);
   }
 
   const ContentPtr
@@ -739,42 +738,6 @@ namespace awkward {
       + FILENAME(__LINE__));
   }
 
-  const ContentPtr
-  EmptyArray::getitem_next_jagged(const Index64& slicestarts,
-                                  const Index64& slicestops,
-                                  const SliceVarNewAxis& slicecontent,
-                                  const Slice& tail) const {
-    throw std::runtime_error(
-      std::string("undefined operation: EmptyArray::getitem_next_jagged(varnewaxis)")
-      + FILENAME(__LINE__));
-  }
-
-  const ContentPtr
-  EmptyArray::getitem_next(const SliceVarNewAxis& varnewaxis,
-                           const Slice& tail,
-                           const Index64& advanced) const {
-    util::handle_error(
-      failure("too many dimensions in slice",
-              kSliceNone,
-              kSliceNone,
-              FILENAME_C(__LINE__)),
-      classname(),
-      identities_.get());
-    return ContentPtr(nullptr);  // make Windows compiler happy
-  }
-
-  const SliceJagged64
-  EmptyArray::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    util::handle_error(
-      failure("too many dimensions in slice",
-              kSliceNone,
-              kSliceNone,
-              FILENAME_C(__LINE__)),
-      classname(),
-      identities_.get());
-    return SliceJagged64(Index64(0), SliceItemPtr(nullptr));  // make Windows compiler happy
-  }
-
   const ContentPtr
   EmptyArray::copy_to(kernel::lib ptr_lib) const {
     IdentitiesPtr identities(nullptr);
