@@ -132,6 +132,10 @@ namespace awkward {
 
   int64_t
   ListForm::purelist_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return 1;
+    }
     return content_.get()->purelist_depth() + 1;
   }
 
@@ -142,6 +146,10 @@ namespace awkward {
 
   const std::pair<int64_t, int64_t>
   ListForm::minmax_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<int64_t, int64_t>(1, 1);
+    }
     std::pair<int64_t, int64_t> content_depth = content_.get()->minmax_depth();
     return std::pair<int64_t, int64_t>(content_depth.first + 1,
                                        content_depth.second + 1);
@@ -149,6 +157,10 @@ namespace awkward {
 
   const std::pair<bool, int64_t>
   ListForm::branch_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<bool, int64_t>(false, 1);
+    }
     std::pair<bool, int64_t> content_depth = content_.get()->branch_depth();
     return std::pair<bool, int64_t>(content_depth.first,
                                     content_depth.second + 1);
@@ -468,7 +480,7 @@ namespace awkward {
         reinterpret_cast<Identities32*>(newidentities.get());
       struct Error err = kernel::new_Identities<int32_t>(
         kernel::lib::cpu,   // DERIVE
-        rawidentities->ptr().get(),
+        rawidentities->data(),
         length());
       util::handle_error(err, classname(), identities_.get());
       setidentities(newidentities);
@@ -482,7 +494,7 @@ namespace awkward {
         reinterpret_cast<Identities64*>(newidentities.get());
       struct Error err = kernel::new_Identities<int64_t>(
         kernel::lib::cpu,   // DERIVE
-        rawidentities->ptr().get(),
+        rawidentities->data(),
         length());
       util::handle_error(err, classname(), identities_.get());
       setidentities(newidentities);
@@ -827,12 +839,20 @@ namespace awkward {
   template <typename T>
   int64_t
   ListArrayOf<T>::purelist_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return 1;
+    }
     return content_.get()->purelist_depth() + 1;
   }
 
   template <typename T>
   const std::pair<int64_t, int64_t>
   ListArrayOf<T>::minmax_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<int64_t, int64_t>(1, 1);
+    }
     std::pair<int64_t, int64_t> content_depth = content_.get()->minmax_depth();
     return std::pair<int64_t, int64_t>(content_depth.first + 1,
                                        content_depth.second + 1);
@@ -841,6 +861,10 @@ namespace awkward {
   template <typename T>
   const std::pair<bool, int64_t>
   ListArrayOf<T>::branch_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<bool, int64_t>(false, 1);
+    }
     std::pair<bool, int64_t> content_depth = content_.get()->branch_depth();
     return std::pair<bool, int64_t>(content_depth.first,
                                     content_depth.second + 1);
@@ -895,7 +919,15 @@ namespace awkward {
       starts_.length(),
       content_.get()->length());
     if (err.str == nullptr) {
-      return content_.get()->validityerror(path + std::string(".content"));
+      if (parameter_equals("__array__", "\"string\"")  ||
+          parameter_equals("__array__", "\"bytestring\"")) {
+        // The content has already been checked and we don't want to trigger the
+        // unnested-char/byte error.
+        return std::string("");
+      }
+      else {
+        return content_.get()->validityerror(path + std::string(".content"));
+      }
     }
     else {
       return (std::string("at ") + path + std::string(" (") + classname()
@@ -1426,6 +1458,14 @@ namespace awkward {
     }
 
     else if (posaxis == depth + 1) {
+      if (parameter_equals("__array__", "\"string\"")  ||
+          parameter_equals("__array__", "\"bytestring\"")) {
+        throw std::invalid_argument(
+          std::string("ak.combinations does not compute combinations of the "
+                      "characters of a string; please split it into lists")
+          + FILENAME(__LINE__));
+      }
+
       int64_t totallen;
       Index64 offsets(length() + 1);
       struct Error err1 = kernel::ListArray_combinations_length_64<T>(
@@ -1508,33 +1548,31 @@ namespace awkward {
                             const Index64& parents,
                             int64_t outlength,
                             bool ascending,
-                            bool stable,
-                            bool keepdims) const {
+                            bool stable) const {
     return toListOffsetArray64(true).get()->sort_next(negaxis,
                                                       starts,
                                                       parents,
                                                       outlength,
                                                       ascending,
-                                                      stable,
-                                                      keepdims);
+                                                      stable);
   }
 
   template <typename T>
   const ContentPtr
   ListArrayOf<T>::argsort_next(int64_t negaxis,
                                const Index64& starts,
+                               const Index64& shifts,
                                const Index64& parents,
                                int64_t outlength,
                                bool ascending,
-                               bool stable,
-                               bool keepdims) const {
+                               bool stable) const {
     return toListOffsetArray64(true).get()->argsort_next(negaxis,
                                                          starts,
+                                                         shifts,
                                                          parents,
                                                          outlength,
                                                          ascending,
-                                                         stable,
-                                                         keepdims);
+                                                         stable);
   }
 
   template <typename T>
@@ -1781,14 +1819,12 @@ namespace awkward {
                                       const Index64& slicestops,
                                       const SliceArray64& slicecontent,
                                       const Slice& tail) const {
-    if (starts_.length() < slicestarts.length()) {
-      util::handle_error(
-        failure("jagged slice length differs from array length",
-                kSliceNone,
-                kSliceNone,
-                FILENAME_C(__LINE__)),
-        classname(),
-        identities_.get());
+    if (slicestarts.length() != length()) {
+      throw std::invalid_argument(
+        std::string("cannot fit jagged slice with length ")
+        + std::to_string(slicestarts.length()) + std::string(" into ")
+        + classname() + std::string(" of size ") + std::to_string(length())
+        + FILENAME(__LINE__));
     }
     if (stops_.length() < starts_.length()) {
       util::handle_error(
@@ -1843,6 +1879,13 @@ namespace awkward {
                                       const Index64& slicestops,
                                       const SliceMissing64& slicecontent,
                                       const Slice& tail) const {
+    if (slicestarts.length() != length()) {
+      throw std::invalid_argument(
+        std::string("cannot fit jagged slice with length ")
+        + std::to_string(slicestarts.length()) + std::string(" into ")
+        + classname() + std::string(" of size ") + std::to_string(length())
+        + FILENAME(__LINE__));
+    }
     if (starts_.length() < slicestarts.length()) {
       util::handle_error(
         failure("jagged slice length differs from array length",
@@ -1924,14 +1967,12 @@ namespace awkward {
                                       const Index64& slicestops,
                                       const SliceJagged64& slicecontent,
                                       const Slice& tail) const {
-    if (starts_.length() < slicestarts.length()) {
-      util::handle_error(
-        failure("jagged slice length differs from array length",
-                kSliceNone,
-                kSliceNone,
-                FILENAME_C(__LINE__)),
-        classname(),
-        identities_.get());
+    if (slicestarts.length() != length()) {
+      throw std::invalid_argument(
+        std::string("cannot fit jagged slice with length ")
+        + std::to_string(slicestarts.length()) + std::string(" into ")
+        + classname() + std::string(" of size ") + std::to_string(length())
+        + FILENAME(__LINE__));
     }
 
     Index64 outoffsets(slicestarts.length() + 1);
@@ -1965,42 +2006,6 @@ namespace awkward {
                                                outcontent);
   }
 
-  template <typename T>
-  const ContentPtr
-  ListArrayOf<T>::getitem_next_jagged(const Index64& slicestarts,
-                                      const Index64& slicestops,
-                                      const SliceVarNewAxis& slicecontent,
-                                      const Slice& tail) const {
-    throw std::runtime_error("FIXME ListArrayOf<T>::SliceVarNewAxis");
-
-    SliceJagged64 jagged = varaxis_to_jagged(slicecontent);
-    return getitem_next_jagged(slicestarts, slicestops, jagged, tail);
-  }
-
-  template <typename T>
-  const ContentPtr
-  ListArrayOf<T>::getitem_next(const SliceVarNewAxis& varnewaxis,
-                               const Slice& tail,
-                               const Index64& advanced) const {
-    SliceJagged64 jagged = content_.get()->varaxis_to_jagged(varnewaxis);
-    return getitem_next(jagged, tail, advanced);
-  }
-
-  template <typename T>
-  const SliceJagged64
-  ListArrayOf<T>::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    Index64 offsets = compact_offsets64(true);
-    Index64 nextcarry(offsets.getitem_at_nowrap(offsets.length() - 1));
-    struct Error err = kernel::SliceVarNewAxis_to_SliceJagged64(
-      kernel::lib::cpu,   // DERIVE
-      nextcarry.data(),
-      offsets.data(),
-      offsets.length() - 1);
-    util::handle_error(err, classname(), identities_.get());
-    SliceItemPtr nextcontent = varnewaxis.content().get()->carry(nextcarry);
-    return SliceJagged64(offsets, nextcontent);
-  }
-
   template <typename T>
   const ContentPtr
   ListArrayOf<T>::copy_to(kernel::lib ptr_lib) const {
