@@ -122,6 +122,10 @@ namespace awkward {
 
   int64_t
   ListOffsetForm::purelist_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return 1;
+    }
     return content_.get()->purelist_depth() + 1;
   }
 
@@ -132,6 +136,10 @@ namespace awkward {
 
   const std::pair<int64_t, int64_t>
   ListOffsetForm::minmax_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<int64_t, int64_t>(1, 1);
+    }
     std::pair<int64_t, int64_t> content_depth = content_.get()->minmax_depth();
     return std::pair<int64_t, int64_t>(content_depth.first + 1,
                                        content_depth.second + 1);
@@ -139,6 +147,10 @@ namespace awkward {
 
   const std::pair<bool, int64_t>
   ListOffsetForm::branch_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<bool, int64_t>(false, 1);
+    }
     std::pair<bool, int64_t> content_depth = content_.get()->branch_depth();
     return std::pair<bool, int64_t>(content_depth.first,
                                     content_depth.second + 1);
@@ -378,11 +390,24 @@ namespace awkward {
   template <typename T>
   const ContentPtr
   ListOffsetArrayOf<T>::toListOffsetArray64(bool start_at_zero) const {
-    if (std::is_same<T, int64_t>::value  &&
-        (!start_at_zero  ||
-         offsets_.getitem_at_nowrap(0) == 0)) {
-      return shallow_copy();
+    if (std::is_same<T, int64_t>::value) {
+      if (!start_at_zero) {
+        return shallow_copy();
+      }
+
+      int64_t offsets0 = offsets_.getitem_at_nowrap(0);
+      if (offsets0 == 0) {
+        return shallow_copy();
+      }
+
+      Index64 offsets = compact_offsets64(start_at_zero);
+      ContentPtr content = content_.get()->getitem_range_nowrap(offsets0, content_.get()->length());
+      return std::make_shared<ListOffsetArrayOf<int64_t>>(identities_,
+                                                          parameters_,
+                                                          offsets,
+                                                          content);
     }
+
     else {
       Index64 offsets = compact_offsets64(start_at_zero);
       return broadcast_tooffsets64(offsets);
@@ -488,7 +513,7 @@ namespace awkward {
         reinterpret_cast<Identities32*>(newidentities.get());
       struct Error err = kernel::new_Identities<int32_t>(
         kernel::lib::cpu,   // DERIVE
-        rawidentities->ptr().get(),
+        rawidentities->data(),
         length());
       util::handle_error(err, classname(), identities_.get());
       setidentities(newidentities);
@@ -503,7 +528,7 @@ namespace awkward {
         reinterpret_cast<Identities64*>(newidentities.get());
       struct Error err = kernel::new_Identities<int64_t>(
         kernel::lib::cpu,   // DERIVE
-        rawidentities->ptr().get(),
+        rawidentities->data(),
         length());
       util::handle_error(err, classname(), identities_.get());
       setidentities(newidentities);
@@ -825,12 +850,20 @@ namespace awkward {
   template <typename T>
   int64_t
   ListOffsetArrayOf<T>::purelist_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return 1;
+    }
     return content_.get()->purelist_depth() + 1;
   }
 
   template <typename T>
   const std::pair<int64_t, int64_t>
   ListOffsetArrayOf<T>::minmax_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<int64_t, int64_t>(1, 1);
+    }
     std::pair<int64_t, int64_t> content_depth = content_.get()->minmax_depth();
     return std::pair<int64_t, int64_t>(content_depth.first + 1,
                                        content_depth.second + 1);
@@ -839,6 +872,10 @@ namespace awkward {
   template <typename T>
   const std::pair<bool, int64_t>
   ListOffsetArrayOf<T>::branch_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<bool, int64_t>(false, 1);
+    }
     std::pair<bool, int64_t> content_depth = content_.get()->branch_depth();
     return std::pair<bool, int64_t>(content_depth.first,
                                     content_depth.second + 1);
@@ -895,7 +932,15 @@ namespace awkward {
       starts.length(),
       content_.get()->length());
     if (err.str == nullptr) {
-      return content_.get()->validityerror(path + std::string(".content"));
+      if (parameter_equals("__array__", "\"string\"")  ||
+          parameter_equals("__array__", "\"bytestring\"")) {
+        // The content has already been checked and we don't want to trigger the
+        // unnested-char/byte error.
+        return std::string("");
+      }
+      else {
+        return content_.get()->validityerror(path + std::string(".content"));
+      }
     }
     else {
       return (std::string("at ") + path + std::string(" (") + classname()
@@ -972,6 +1017,15 @@ namespace awkward {
                                                         offsets_,
                                                         pair.second));
       }
+      else if (offsets_.length() == 1) {
+        Index64 tooffsets = inneroffsets.getitem_range_nowrap(0, 1);
+        return std::pair<Index64, ContentPtr>(
+                 Index64(0),
+                 std::make_shared<ListOffsetArray64>(Identities::none(),
+                                                     util::Parameters(),
+                                                     tooffsets,
+                                                     pair.second));
+      }
       else {
         Index64 tooffsets(offsets_.length());
         struct Error err = kernel::ListOffsetArray_flatten_offsets_64<T>(
@@ -1312,6 +1366,13 @@ namespace awkward {
     int64_t outlength,
     bool mask,
     bool keepdims) const {
+    if (offsets_.getitem_at_nowrap(0) != 0) {
+      ContentPtr next = toListOffsetArray64(true);
+      return next.get()->reduce_next(
+          reducer, negaxis, starts, shifts, parents, outlength, mask, keepdims
+      );
+    }
+
     std::pair<bool, int64_t> branchdepth = branch_depth();
 
     if (!branchdepth.first  &&  negaxis == branchdepth.second) {
@@ -1410,6 +1471,7 @@ namespace awkward {
       }
 
       ContentPtr nextcontent = content_.get()->carry(nextcarry, false);
+
       ContentPtr outcontent = nextcontent.get()->reduce_next(reducer,
                                                              negaxis - 1,
                                                              nextstarts,
@@ -1418,7 +1480,6 @@ namespace awkward {
                                                              maxnextparents + 1,
                                                              mask,
                                                              false);
-
       ContentPtr out = std::make_shared<ListArray64>(Identities::none(),
                                                      util::Parameters(),
                                                      outstarts,
@@ -1438,6 +1499,7 @@ namespace awkward {
     }
 
     else {
+
       int64_t globalstart;
       int64_t globalstop;
       struct Error err1 = kernel::ListOffsetArray_reduce_global_startstop_64(
@@ -1560,6 +1622,14 @@ namespace awkward {
     }
 
     else if (posaxis == depth + 1) {
+      if (parameter_equals("__array__", "\"string\"")  ||
+          parameter_equals("__array__", "\"bytestring\"")) {
+        throw std::invalid_argument(
+          std::string("ak.combinations does not compute combinations of the "
+                      "characters of a string; please split it into lists")
+          + FILENAME(__LINE__));
+      }
+
       IndexOf<T> starts = util::make_starts(offsets_);
       IndexOf<T> stops = util::make_stops(offsets_);
 
@@ -1642,15 +1712,13 @@ namespace awkward {
                                   const Index64& parents,
                                   int64_t outlength,
                                   bool ascending,
-                                  bool stable,
-                                  bool keepdims) const {
+                                  bool stable) const {
     return toListOffsetArray64(true).get()->sort_next(negaxis,
                                                       starts,
                                                       parents,
                                                       outlength,
                                                       ascending,
-                                                      stable,
-                                                      keepdims);
+                                                      stable);
   }
 
   template <>
@@ -1660,31 +1728,55 @@ namespace awkward {
     const Index64& parents,
     int64_t outlength,
     bool ascending,
-    bool stable,
-    bool keepdims) const {
+    bool stable) const {
 
     if (length() == 0 ) {
       return shallow_copy();
     }
 
-    // if this is array of strings, axis parameter is ignored
-    // and this array is sorted
-    if (util::parameter_isstring(parameters_, "__array__")) {
-      if (NumpyArray* content = dynamic_cast<NumpyArray*>(content_.get())) {
-        ContentPtr out = content->sort_asstrings(offsets_,
-                                                 ascending,
-                                                 stable);
-        return std::make_shared<RegularArray>(Identities::none(),
-                                              util::Parameters(),
-                                              out,
-                                              out.get()->length(),
-                                              length());
-      }
-    }
-
     std::pair<bool, int64_t> branchdepth = branch_depth();
 
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      if (branchdepth.first  ||  negaxis != branchdepth.second) {
+        throw std::invalid_argument(
+          std::string("array with strings can only be sorted with axis=-1")
+          + FILENAME(__LINE__));
+      }
+
+      std::string validity_error = validityerror("");
+      if (!validity_error.empty()) {
+        throw std::invalid_argument(validity_error + FILENAME(__LINE__));
+      }
+      NumpyArray* rawcontent = dynamic_cast<NumpyArray*>(content_.get());
+
+      Index64 tocarry(parents.length());
+      struct Error err = kernel::ListOffsetArray_argsort_strings(
+        kernel::lib::cpu,   // DERIVE
+        tocarry.data(),
+        parents.data(),
+        parents.length(),
+        reinterpret_cast<uint8_t*>(rawcontent->data()),
+        util::make_starts(offsets_).data(),
+        util::make_stops(offsets_).data(),
+        stable,
+        ascending,
+        false);
+      util::handle_error(err, classname(), identities_.get());
+
+      ContentPtr out = carry(tocarry, false);
+
+      return out;
+    }
+
     if (!branchdepth.first  &&  negaxis == branchdepth.second) {
+      if (purelist_parameter("__array__") == "\"string\""  ||
+          purelist_parameter("__array__") == "\"bytestring\"") {
+        throw std::invalid_argument(
+          std::string("array with strings can only be sorted with axis=-1")
+          + FILENAME(__LINE__));
+      }
+
       if (offsets_.length() - 1 != parents.length()) {
         throw std::runtime_error(
           std::string("offsets_.length() - 1 != parents.length()" + FILENAME(__LINE__)));
@@ -1741,7 +1833,7 @@ namespace awkward {
 
       ContentPtr outcontent = nextcontent.get()->sort_next(
         negaxis - 1, nextstarts, nextparents, nextcontent.get()->length(),
-        ascending, stable, false);
+        ascending, stable);
 
       Index64 outcarry(nextlen);
       struct Error err5 = kernel::ListOffsetArray_local_preparenext_64(
@@ -1753,17 +1845,11 @@ namespace awkward {
 
       outcontent = outcontent.get()->carry(outcarry, false);
 
+      Index64 outoffsets = compact_offsets64(true);
       ContentPtr out = std::make_shared<ListOffsetArray64>(Identities::none(),
                                                            parameters_,
-                                                           offsets_,
+                                                           outoffsets,
                                                            outcontent);
-      if (keepdims) {
-        out = std::make_shared<RegularArray>(Identities::none(),
-                                             util::Parameters(),
-                                             out,
-                                             out.get()->length(),
-                                             length());
-      }
       return out;
     }
     else {
@@ -1790,19 +1876,13 @@ namespace awkward {
 
       ContentPtr outcontent = trimmed.get()->sort_next(
         negaxis, util::make_starts(offsets_), nextparents,
-        offsets_.length() - 1, ascending, stable, false);
+        offsets_.length() - 1, ascending, stable);
 
+      Index64 outoffsets = compact_offsets64(true);
       ContentPtr out = std::make_shared<ListOffsetArray64>(Identities::none(),
                                                            parameters_,
-                                                           offsets_,
+                                                           outoffsets,
                                                            outcontent);
-      if (keepdims) {
-        out = std::make_shared<RegularArray>(Identities::none(),
-                                             util::Parameters(),
-                                             out,
-                                             out.get()->length(),
-                                             length());
-      }
       return out;
     }
   }
@@ -1811,42 +1891,76 @@ namespace awkward {
   const ContentPtr
   ListOffsetArrayOf<T>::argsort_next(int64_t negaxis,
                                      const Index64& starts,
+                                     const Index64& shifts,
                                      const Index64& parents,
                                      int64_t outlength,
                                      bool ascending,
-                                     bool stable,
-                                     bool keepdims) const {
+                                     bool stable) const {
     return toListOffsetArray64(true).get()->argsort_next(negaxis,
                                                          starts,
+                                                         shifts,
                                                          parents,
                                                          outlength,
                                                          ascending,
-                                                         stable,
-                                                         keepdims);
+                                                         stable);
   }
 
   template <>
   const ContentPtr
   ListOffsetArrayOf<int64_t>::argsort_next(int64_t negaxis,
                                            const Index64& starts,
+                                           const Index64& shifts,
                                            const Index64& parents,
                                            int64_t outlength,
                                            bool ascending,
-                                           bool stable,
-                                           bool keepdims) const {
+                                           bool stable) const {
     if (length() == 0 ) {
      return shallow_copy();
     }
 
-    // if this is array of strings, axis parameter is ignored
-    // and this array is sorted
-    if (util::parameter_isstring(parameters_, "__array__")) {
-      throw std::runtime_error(
-        std::string("not implemented yet: argsort for strings") + FILENAME(__LINE__));
+    std::pair<bool, int64_t> branchdepth = branch_depth();
+
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      if (branchdepth.first  ||  negaxis != branchdepth.second) {
+        throw std::invalid_argument(
+          std::string("array with strings can only be sorted with axis=-1")
+          + FILENAME(__LINE__));
+      }
+
+      std::string validity_error = validityerror("");
+      if (!validity_error.empty()) {
+        throw std::invalid_argument(validity_error + FILENAME(__LINE__));
+      }
+      NumpyArray* rawcontent = dynamic_cast<NumpyArray*>(content_.get());
+
+      Index64 output(parents.length());
+      struct Error err = kernel::ListOffsetArray_argsort_strings(
+        kernel::lib::cpu,   // DERIVE
+        output.data(),
+        parents.data(),
+        parents.length(),
+        reinterpret_cast<uint8_t*>(rawcontent->data()),
+        util::make_starts(offsets_).data(),
+        util::make_stops(offsets_).data(),
+        stable,
+        ascending,
+        true);
+      util::handle_error(err, classname(), identities_.get());
+
+      ContentPtr out = std::make_shared<NumpyArray>(output);
+
+      return out;
     }
 
-    std::pair<bool, int64_t> branchdepth = branch_depth();
     if (!branchdepth.first  &&  negaxis == branchdepth.second) {
+      if (purelist_parameter("__array__") == "\"string\""  ||
+          purelist_parameter("__array__") == "\"bytestring\"") {
+        throw std::invalid_argument(
+          std::string("array with strings can only be sorted with axis=-1")
+          + FILENAME(__LINE__));
+      }
+
       if (offsets_.length() - 1 != parents.length()) {
         throw std::runtime_error(
           std::string("offsets_.length() - 1 != parents.length()") + FILENAME(__LINE__));
@@ -1900,11 +2014,33 @@ namespace awkward {
         nextlen);
       util::handle_error(err4, classname(), identities_.get());
 
+      Index64 nextshifts(nextlen);
+      Index64 nummissing(maxcount);
+      Index64 missing(offsets_.getitem_at(offsets_.length() - 1));
+      struct Error err7 = kernel::ListOffsetArray_reduce_nonlocal_nextshifts_64(
+        kernel::lib::cpu,   // DERIVE
+        nummissing.data(),
+        missing.data(),
+        nextshifts.data(),
+        offsets_.data(),
+        offsets_.length() - 1,
+        starts.data(),
+        parents.data(),
+        maxcount,
+        nextlen,
+        nextcarry.data());
+      util::handle_error(err7, classname(), identities_.get());
+
       ContentPtr nextcontent = content_.get()->carry(nextcarry, false);
 
       ContentPtr outcontent = nextcontent.get()->argsort_next(
-        negaxis - 1, nextstarts, nextparents, maxnextparents + 1,
-        ascending, stable, false);
+        negaxis - 1,
+        nextstarts,
+        nextshifts,
+        nextparents,
+        maxnextparents + 1,
+        ascending,
+        stable);
 
       Index64 outcarry(nextlen);
       struct Error err5 = kernel::ListOffsetArray_local_preparenext_64(
@@ -1916,17 +2052,11 @@ namespace awkward {
 
       outcontent = outcontent.get()->carry(outcarry, false);
 
+      Index64 outoffsets = compact_offsets64(true);;
       ContentPtr out = std::make_shared<ListOffsetArray64>(Identities::none(),
                                                            util::Parameters(),
-                                                           offsets_,
+                                                           outoffsets,
                                                            outcontent);
-      if (keepdims) {
-        out = std::make_shared<RegularArray>(Identities::none(),
-                                             util::Parameters(),
-                                             out,
-                                             out.get()->length(),
-                                             length());
-      }
       return out;
     }
     else {
@@ -1950,22 +2080,20 @@ namespace awkward {
 
       ContentPtr trimmed = content_.get()->getitem_range_nowrap(globalstart,
                                                                 globalstop);
-
       ContentPtr outcontent = trimmed.get()->argsort_next(
-        negaxis, util::make_starts(offsets_), nextparents,
-        offsets_.length() - 1, ascending, stable, false);
+        negaxis,
+        util::make_starts(offsets_),
+        shifts,
+        nextparents,
+        offsets_.length() - 1,
+        ascending,
+        stable);
 
+      Index64 outoffsets = compact_offsets64(true);
       ContentPtr out = std::make_shared<ListOffsetArray64>(Identities::none(),
                                                            util::Parameters(),
-                                                           offsets_,
+                                                           outoffsets,
                                                            outcontent);
-      if (keepdims) {
-        out = std::make_shared<RegularArray>(Identities::none(),
-                                             util::Parameters(),
-                                             out,
-                                             out.get()->length(),
-                                             length());
-      }
       return out;
     }
   }
@@ -2196,57 +2324,6 @@ namespace awkward {
                                          tail);
   }
 
-  template <typename T>
-  const ContentPtr
-  ListOffsetArrayOf<T>::getitem_next_jagged(const Index64& slicestarts,
-                                            const Index64& slicestops,
-                                            const SliceVarNewAxis& slicecontent,
-                                            const Slice& tail) const {
-    ListArrayOf<T> listarray(identities_,
-                             parameters_,
-                             util::make_starts(offsets_),
-                             util::make_stops(offsets_),
-                             content_);
-    return listarray.getitem_next_jagged(slicestarts,
-                                         slicestops,
-                                         slicecontent,
-                                         tail);
-  }
-
-  template <typename T>
-  const ContentPtr
-  ListOffsetArrayOf<T>::getitem_next(const SliceVarNewAxis& varnewaxis,
-                                     const Slice& tail,
-                                     const Index64& advanced) const {
-    SliceJagged64 jagged = content_.get()->varaxis_to_jagged(varnewaxis);
-    return getitem_next(jagged, tail, advanced);
-  }
-
-  template <typename T>
-  const SliceJagged64
-  ListOffsetArrayOf<T>::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    Index64 offsets = compact_offsets64(true);
-    Index64 nextcarry(offsets.getitem_at_nowrap(offsets.length() - 1));
-
-
-    // FIXME: to kernel
-    int64_t* tocarry = nextcarry.data();
-    const int64_t* fromoffsets = offsets.data();
-    int64_t len = offsets.length() - 1;
-    for (int64_t i = 0;  i < len;  i++) {
-      int64_t start = fromoffsets[i];
-      int64_t stop = fromoffsets[i + 1];
-      for (int64_t j = start;  j < stop;  j++) {
-        tocarry[j] = i;
-      }
-    }
-
-
-    SliceItemPtr nextcontent = varnewaxis.content().get()->carry(nextcarry);
-
-    return SliceJagged64(offsets, nextcontent);
-  }
-
   template <typename T>
   const ContentPtr
   ListOffsetArrayOf<T>::copy_to(kernel::lib ptr_lib) const {
@@ -2296,7 +2373,8 @@ namespace awkward {
 
   template <>
   bool ListOffsetArrayOf<int64_t>::is_unique() const {
-    if (util::parameter_isstring(parameters_, "__array__")) {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
       if (NumpyArray* content = dynamic_cast<NumpyArray*>(content_.get())) {
         ContentPtr out = content->as_unique_strings(offsets_);
         return (out.get()->length() == length());
