@@ -290,14 +290,14 @@ namespace awkward {
   const SliceItemPtr
   None::asslice() const {
     throw std::runtime_error(
-      std::string("undefined opteration: None::asslice")
+      std::string("undefined operation: None::asslice")
       + FILENAME(__LINE__));
   }
 
   const ContentPtr
   None::fillna(const ContentPtr& value) const {
     throw std::runtime_error(
-      std::string("undefined opteration: None::fillna")
+      std::string("undefined operation: None::fillna")
       + FILENAME(__LINE__));
   }
 
@@ -354,8 +354,7 @@ namespace awkward {
                   const Index64& parents,
                   int64_t outlength,
                   bool ascending,
-                  bool stable,
-                  bool keepdims) const {
+                  bool stable) const {
     throw std::runtime_error(
       std::string("undefined operation: None::sort_next")
       + FILENAME(__LINE__));
@@ -364,11 +363,11 @@ namespace awkward {
   const ContentPtr
   None::argsort_next(int64_t negaxis,
                      const Index64& starts,
+                     const Index64& shifts,
                      const Index64& parents,
                      int64_t outlength,
                      bool ascending,
-                     bool stable,
-                     bool keepdims) const {
+                     bool stable) const {
     throw std::runtime_error(
       std::string("undefined operation: None::argsort_next")
       + FILENAME(__LINE__));
@@ -458,32 +457,6 @@ namespace awkward {
       + FILENAME(__LINE__));
   }
 
-  const ContentPtr
-  None::getitem_next_jagged(const Index64& slicestarts,
-                            const Index64& slicestops,
-                            const SliceVarNewAxis& slicecontent,
-                            const Slice& tail) const {
-    throw std::runtime_error(
-      std::string("undefined operation: None::getitem_next_jagged(varnewaxis)")
-      + FILENAME(__LINE__));
-  }
-
-  const ContentPtr
-  None::getitem_next(const SliceVarNewAxis& varnewaxis,
-                     const Slice& tail,
-                     const Index64& advanced) const {
-    throw std::runtime_error(
-      std::string("undefined operation: None::getitem_next(varnewaxis)")
-      + FILENAME(__LINE__));
-  }
-
-  const SliceJagged64
-  None::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    throw std::runtime_error(
-      std::string("undefined operation: None::varaxis_to_jagged(varnewaxis)")
-      + FILENAME(__LINE__));
-  }
-
   const ContentPtr
   None::copy_to(kernel::lib ptr_lib) const {
     throw std::runtime_error(
