@@ -512,34 +512,32 @@ namespace awkward {
                     const Index64& parents,
                     int64_t outlength,
                     bool ascending,
-                    bool stable,
-                    bool keepdims) const {
+                    bool stable) const {
     ContentPtr next = array_.get()->getitem_at_nowrap(at_);
     return next.get()->sort_next(negaxis,
                                  starts,
                                  parents,
                                  outlength,
                                  ascending,
-                                 stable,
-                                 keepdims);
+                                 stable);
   }
 
   const ContentPtr
   Record::argsort_next(int64_t negaxis,
                        const Index64& starts,
+                       const Index64& shifts,
                        const Index64& parents,
                        int64_t outlength,
                        bool ascending,
-                       bool stable,
-                       bool keepdims) const {
+                       bool stable) const {
     ContentPtr next = array_.get()->getitem_at_nowrap(at_);
     return next.get()->argsort_next(negaxis,
                                     starts,
+                                    shifts,
                                     parents,
                                     outlength,
                                     ascending,
-                                    stable,
-                                    keepdims);
+                                    stable);
   }
 
   const ContentPtr
@@ -645,32 +643,6 @@ namespace awkward {
       + FILENAME(__LINE__));
   }
 
-  const ContentPtr
-  Record::getitem_next_jagged(const Index64& slicestarts,
-                              const Index64& slicestops,
-                              const SliceVarNewAxis& slicecontent,
-                              const Slice& tail) const {
-    throw std::runtime_error(
-      std::string("undefined operation: Record::getitem_next_jagged(varnewaxis)")
-      + FILENAME(__LINE__));
-  }
-
-  const ContentPtr
-  Record::getitem_next(const SliceVarNewAxis& varnewaxis,
-                       const Slice& tail,
-                       const Index64& advanced) const {
-    throw std::runtime_error(
-      std::string("undefined operation: Record::getitem_next(varnewaxis)")
-      + FILENAME(__LINE__));
-  }
-
-  const SliceJagged64
-  Record::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    throw std::runtime_error(
-      std::string("undefined operation: Record::varaxis_to_jagged(varnewaxis)")
-      + FILENAME(__LINE__));
-  }
-
   const ContentPtr
   Record::copy_to(kernel::lib ptr_lib) const {
     ContentPtr array = array_.get()->copy_to(ptr_lib);
