@@ -19,6 +19,7 @@
 #include "awkward/array/BitMaskedArray.h"
 #include "awkward/array/UnmaskedArray.h"
 #include "awkward/array/NumpyArray.h"
+#include "awkward/array/RegularArray.h"
 #include "awkward/array/VirtualArray.h"
 
 #include "awkward/array/RecordArray.h"
@@ -655,10 +656,7 @@ namespace awkward {
                              const std::string& pre,
                              const std::string& post) const {
     std::stringstream out;
-    out << indent << pre << "<" << classname();
-    if (contents_.empty()) {
-      out << " length=\"" << length_ << "\"";
-    }
+    out << indent << pre << "<" << classname() << " length=\"" << length_ << "\"";
     out << ">\n";
     if (identities_.get() != nullptr) {
       out << identities_.get()->tostring_part(
@@ -944,7 +942,7 @@ namespace awkward {
     }
     if (allow_lazy) {
       return std::make_shared<IndexedArray64>(identities,
-                                              parameters_,
+                                              util::Parameters(),
                                               carry,
                                               shallow_copy());
     }
@@ -1608,8 +1606,7 @@ namespace awkward {
                          const Index64& parents,
                          int64_t outlength,
                          bool ascending,
-                         bool stable,
-                         bool keepdims) const {
+                         bool stable) const {
     if (length() == 0) {
       return shallow_copy();
     }
@@ -1621,38 +1618,50 @@ namespace awkward {
                                                  parents,
                                                  outlength,
                                                  ascending,
-                                                 stable,
-                                                 keepdims);
+                                                 stable);
+      next = std::make_shared<RegularArray>(
+        Identities::none(),
+        util::Parameters(),
+        next,
+        next.get()->length(),
+        next.get()->length());
+
       contents.push_back(next);
     }
     return std::make_shared<RecordArray>(Identities::none(),
                                          parameters_,
                                          contents,
                                          recordlookup_,
-                                         outlength);
+                                         outlength).get()->getitem_at_nowrap(0);
   }
 
   const ContentPtr
   RecordArray::argsort_next(int64_t negaxis,
                             const Index64& starts,
+                            const Index64& shifts,
                             const Index64& parents,
                             int64_t outlength,
                             bool ascending,
-                            bool stable,
-                            bool keepdims) const {
+                            bool stable) const {
     if (length() == 0) {
-      return shallow_copy();
+      return std::make_shared<NumpyArray>(Index64(0));
     }
     std::vector<ContentPtr> contents;
     for (auto content : contents_) {
       ContentPtr trimmed = content.get()->getitem_range_nowrap(0, length());
       ContentPtr next = trimmed.get()->argsort_next(negaxis,
                                                     starts,
+                                                    shifts,
                                                     parents,
                                                     outlength,
                                                     ascending,
-                                                    stable,
-                                                    keepdims);
+                                                    stable);
+      next = std::make_shared<RegularArray>(
+        Identities::none(),
+        util::Parameters(),
+        next,
+        next.get()->length(),
+        next.get()->length());
       contents.push_back(next);
     }
     return std::make_shared<RecordArray>(
@@ -1660,7 +1669,7 @@ namespace awkward {
       util::Parameters(),
       contents,
       recordlookup_,
-      outlength);
+      outlength).get()->getitem_at_nowrap(0);
   }
 
   const ContentPtr
@@ -1796,33 +1805,6 @@ namespace awkward {
                                                       tail);
   }
 
-  const ContentPtr
-  RecordArray::getitem_next_jagged(const Index64& slicestarts,
-                                   const Index64& slicestops,
-                                   const SliceVarNewAxis& slicecontent,
-                                   const Slice& tail) const {
-    return getitem_next_jagged_generic<SliceVarNewAxis>(slicestarts,
-                                                        slicestops,
-                                                        slicecontent,
-                                                        tail);
-  }
-
-  const ContentPtr
-  RecordArray::getitem_next(const SliceVarNewAxis& varnewaxis,
-                            const Slice& tail,
-                            const Index64& advanced) const {
-    throw std::invalid_argument(
-      std::string("cannot slice records by a newaxis (length-1 regular dimension)")
-      + FILENAME(__LINE__));
-  }
-
-  const SliceJagged64
-  RecordArray::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    throw std::invalid_argument(
-      std::string("cannot slice records by a newaxis (length-1 regular dimension)")
-      + FILENAME(__LINE__));
-  }
-
   const ContentPtr
   RecordArray::copy_to(kernel::lib ptr_lib) const {
     ContentPtrVec contents;
@@ -1870,7 +1852,8 @@ namespace awkward {
     else {
       ContentPtrVec contents;
       for (auto content : contents_) {
-        contents.push_back(content.get()->getitem_next_jagged(slicestarts,
+        ContentPtr trimmed = content.get()->getitem_range_nowrap(0, length());
+        contents.push_back(trimmed.get()->getitem_next_jagged(slicestarts,
                                                               slicestops,
                                                               slicecontent,
                                                               tail));
