@@ -109,6 +109,10 @@ namespace awkward {
 
   int64_t
   RegularForm::purelist_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return 1;
+    }
     return content_.get()->purelist_depth() + 1;
   }
 
@@ -119,6 +123,10 @@ namespace awkward {
 
   const std::pair<int64_t, int64_t>
   RegularForm::minmax_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<int64_t, int64_t>(1, 1);
+    }
     std::pair<int64_t, int64_t> content_depth = content_.get()->minmax_depth();
     return std::pair<int64_t, int64_t>(content_depth.first + 1,
                                        content_depth.second + 1);
@@ -126,6 +134,10 @@ namespace awkward {
 
   const std::pair<bool, int64_t>
   RegularForm::branch_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<bool, int64_t>(false, 1);
+    }
     std::pair<bool, int64_t> content_depth = content_.get()->branch_depth();
     return std::pair<bool, int64_t>(content_depth.first,
                                     content_depth.second + 1);
@@ -717,11 +729,19 @@ namespace awkward {
 
   int64_t
   RegularArray::purelist_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return 1;
+    }
     return content_.get()->purelist_depth() + 1;
   }
 
   const std::pair<int64_t, int64_t>
   RegularArray::minmax_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<int64_t, int64_t>(1, 1);
+    }
     std::pair<int64_t, int64_t> content_depth = content_.get()->minmax_depth();
     return std::pair<int64_t, int64_t>(content_depth.first + 1,
                                        content_depth.second + 1);
@@ -729,6 +749,10 @@ namespace awkward {
 
   const std::pair<bool, int64_t>
   RegularArray::branch_depth() const {
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      return std::pair<bool, int64_t>(false, 1);
+    }
     std::pair<bool, int64_t> content_depth = content_.get()->branch_depth();
     return std::pair<bool, int64_t>(content_depth.first,
                                     content_depth.second + 1);
@@ -770,7 +794,15 @@ namespace awkward {
               + std::string("): ") + std::string("size < 0")
               + FILENAME(__LINE__));
     }
-    return content_.get()->validityerror(path + std::string(".content"));
+    if (parameter_equals("__array__", "\"string\"")  ||
+        parameter_equals("__array__", "\"bytestring\"")) {
+      // The content has already been checked and we don't want to trigger the
+      // unnested-char/byte error.
+      return std::string("");
+    }
+    else {
+      return content_.get()->validityerror(path + std::string(".content"));
+    }
   }
 
   const ContentPtr
@@ -935,15 +967,10 @@ namespace awkward {
 
   const SliceItemPtr
   RegularArray::asslice() const {
-    if (size_ == 1) {
-      return std::make_shared<SliceVarNewAxis>(content_.get()->asslice());
-    }
-    else {
-      throw std::invalid_argument(
-        std::string("slice items can have all fixed-size dimensions (to follow NumPy's "
-                    "slice rules) or they can have all var-sized dimensions (for jagged "
-                    "indexing), but not both in the same slice item") + FILENAME(__LINE__));
-    }
+    throw std::invalid_argument(
+      std::string("slice items can have all fixed-size dimensions (to follow NumPy's "
+                  "slice rules) or they can have all var-sized dimensions (for jagged "
+                  "indexing), but not both in the same slice item") + FILENAME(__LINE__));
   }
 
   const ContentPtr
@@ -1142,6 +1169,14 @@ namespace awkward {
     }
 
     else if (posaxis == depth + 1) {
+      if (parameter_equals("__array__", "\"string\"")  ||
+          parameter_equals("__array__", "\"bytestring\"")) {
+        throw std::invalid_argument(
+          std::string("ak.combinations does not compute combinations of the "
+                      "characters of a string; please split it into lists")
+          + FILENAME(__LINE__));
+      }
+
       int64_t size = size_;
       if (replacement) {
         size += (n - 1);
@@ -1238,8 +1273,7 @@ namespace awkward {
                           const Index64& parents,
                           int64_t outlength,
                           bool ascending,
-                          bool stable,
-                          bool keepdims) const {
+                          bool stable) const {
     if (length() == 0) {
       return shallow_copy();
     }
@@ -1249,8 +1283,7 @@ namespace awkward {
                                        parents,
                                        outlength,
                                        ascending,
-                                       stable,
-                                       keepdims);
+                                       stable);
     if (RegularArray* raw1 =
             dynamic_cast<RegularArray*>(out.get())) {
       if (ListOffsetArray64* raw2 =
@@ -1269,22 +1302,22 @@ namespace awkward {
   const ContentPtr
   RegularArray::argsort_next(int64_t negaxis,
                              const Index64& starts,
+                             const Index64& shifts,
                              const Index64& parents,
                              int64_t outlength,
                              bool ascending,
-                             bool stable,
-                             bool keepdims) const {
+                             bool stable) const {
     if (length() == 0) {
-      return shallow_copy();
+      return std::make_shared<NumpyArray>(Index64(0));
     }
     std::shared_ptr<Content> out = toListOffsetArray64(true).get()->argsort_next(
                                        negaxis,
                                        starts,
+                                       shifts,
                                        parents,
                                        outlength,
                                        ascending,
-                                       stable,
-                                       keepdims);
+                                       stable);
     if (RegularArray* raw1 =
             dynamic_cast<RegularArray*>(out.get())) {
       if (ListOffsetArray64* raw2 =
@@ -1348,6 +1381,7 @@ namespace awkward {
                                   range.start() != Slice::none(),
                                   range.stop() != Slice::none(),
                                   size_);
+
     int64_t nextsize = 0;
     if (range.step() > 0  &&  regular_stop - regular_start > 0) {
       int64_t diff = regular_stop - regular_start;
@@ -1558,50 +1592,6 @@ namespace awkward {
                                            tail);
   }
 
-  const ContentPtr
-  RegularArray::getitem_next_jagged(const Index64& slicestarts,
-                                    const Index64& slicestops,
-                                    const SliceVarNewAxis& slicecontent,
-                                    const Slice& tail) const {
-    ContentPtr self = toListOffsetArray64(true);
-    return self.get()->getitem_next_jagged(slicestarts,
-                                           slicestops,
-                                           slicecontent,
-                                           tail);
-  }
-
-  const ContentPtr
-  RegularArray::getitem_next(const SliceVarNewAxis& varnewaxis,
-                             const Slice& tail,
-                             const Index64& advanced) const {
-    SliceJagged64 jagged = content_.get()->varaxis_to_jagged(varnewaxis);
-    return getitem_next(jagged, tail, advanced);
-  }
-
-  const SliceJagged64
-  RegularArray::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    Index64 offsets = compact_offsets64(true);
-    Index64 nextcarry(offsets.getitem_at_nowrap(offsets.length() - 1));
-
-
-    // FIXME: to kernel
-    int64_t* tocarry = nextcarry.data();
-    const int64_t* fromoffsets = offsets.data();
-    int64_t len = offsets.length() - 1;
-    for (int64_t i = 0;  i < len;  i++) {
-      int64_t start = fromoffsets[i];
-      int64_t stop = fromoffsets[i + 1];
-      for (int64_t j = start;  j < stop;  j++) {
-        tocarry[j] = i;
-      }
-    }
-
-
-    SliceItemPtr nextcontent = varnewaxis.content().get()->carry(nextcarry);
-
-    return SliceJagged64(offsets, nextcontent);
-  }
-
   const ContentPtr
   RegularArray::copy_to(kernel::lib ptr_lib) const {
     ContentPtr content = content_.get()->copy_to(ptr_lib);
