@@ -131,7 +131,7 @@ namespace awkward {
       }
       out = contents_[0].get()->purelist_parameter(key);
       for (size_t i = 1;  i < contents_.size();  i++) {
-        if (!contents_[i].get()->parameter_equals(key, out)) {
+        if (!util::json_equals(out, contents_[i].get()->purelist_parameter(key))) {
           return "null";
         }
       }
@@ -514,6 +514,7 @@ namespace awkward {
       lentags,
       index);
     util::handle_error(err, classname(), identities_.get());
+
     Index64 nextcarry(tmpcarry.ptr(), 0, lenout, tmpcarry.ptr_lib());
     return contents_[(size_t)index].get()->carry(nextcarry, false);
   }
@@ -1172,11 +1173,11 @@ namespace awkward {
     for (auto content : contents_) {
       contents.push_back(content.get()->getitem_field(key));
     }
-    return std::make_shared<UnionArrayOf<T, I>>(identities_,
-                                                util::Parameters(),
-                                                tags_,
-                                                index_,
-                                                contents);
+    return UnionArrayOf<T, I>(identities_,
+                              util::Parameters(),
+                              tags_,
+                              index_,
+                              contents).simplify_uniontype(true, false);
   }
 
   template <typename T, typename I>
@@ -1187,11 +1188,11 @@ namespace awkward {
     for (auto content : contents_) {
       contents.push_back(content.get()->getitem_field(key, only_fields));
     }
-    return std::make_shared<UnionArrayOf<T, I>>(identities_,
-                                                util::Parameters(),
-                                                tags_,
-                                                index_,
-                                                contents);
+    return UnionArrayOf<T, I>(identities_,
+                              util::Parameters(),
+                              tags_,
+                              index_,
+                              contents).simplify_uniontype(true, false);
   }
 
   template <typename T, typename I>
@@ -1202,11 +1203,11 @@ namespace awkward {
     for (auto content : contents_) {
       contents.push_back(content.get()->getitem_fields(keys));
     }
-    return std::make_shared<UnionArrayOf<T, I>>(identities_,
-                                                util::Parameters(),
-                                                tags_,
-                                                index_,
-                                                contents);
+    return UnionArrayOf<T, I>(identities_,
+                              util::Parameters(),
+                              tags_,
+                              index_,
+                              contents).simplify_uniontype(true, false);
   }
 
   template <typename T, typename I>
@@ -1218,11 +1219,11 @@ namespace awkward {
     for (auto content : contents_) {
       contents.push_back(content.get()->getitem_fields(keys, only_fields));
     }
-    return std::make_shared<UnionArrayOf<T, I>>(identities_,
-                                                util::Parameters(),
-                                                tags_,
-                                                index_,
-                                                contents);
+    return UnionArrayOf<T, I>(identities_,
+                              util::Parameters(),
+                              tags_,
+                              index_,
+                              contents).simplify_uniontype(true, false);
   }
 
   template <typename T, typename I>
@@ -1271,10 +1272,6 @@ namespace awkward {
              dynamic_cast<SliceMissing64*>(head.get())) {
       return Content::getitem_next(*missing, tail, advanced);
     }
-    else if (SliceVarNewAxis* varnewaxis =
-             dynamic_cast<SliceVarNewAxis*>(head.get())) {
-      return UnionArrayOf<T, I>::getitem_next(*varnewaxis, tail, advanced);
-    }
     else {
       throw std::runtime_error(
         std::string("unrecognized slice type") + FILENAME(__LINE__));
@@ -1454,6 +1451,14 @@ namespace awkward {
     if (paramcheck != std::string("")) {
       return paramcheck;
     }
+    for (int64_t i = 0;  i < numcontents();  i++) {
+      if (dynamic_cast<UnionArray8_32*>(content(i).get())  ||
+          dynamic_cast<UnionArray8_U32*>(content(i).get())  ||
+          dynamic_cast<UnionArray8_64*>(content(i).get())) {
+        return classname() + " contains " + content(i).get()->classname() +
+               ", the operation that made it might have forgotten to call 'simplify_uniontype()'";
+      }
+    }
     if (index_.length() < tags_.length()) {
       return (std::string("at ") + path + std::string(" (") + classname()
               + std::string("): ") + std::string("len(index) < len(tags)")
@@ -2089,8 +2094,7 @@ namespace awkward {
                                 const Index64& parents,
                                 int64_t outlength,
                                 bool ascending,
-                                bool stable,
-                                bool keepdims) const {
+                                bool stable) const {
     if (length() == 0) {
       return shallow_copy();
     }
@@ -2106,21 +2110,20 @@ namespace awkward {
                                        parents,
                                        outlength,
                                        ascending,
-                                       stable,
-                                       keepdims);
+                                       stable);
   }
 
   template <typename T, typename I>
   const ContentPtr
   UnionArrayOf<T, I>::argsort_next(int64_t negaxis,
                                    const Index64& starts,
+                                   const Index64& shifts,
                                    const Index64& parents,
                                    int64_t outlength,
                                    bool ascending,
-                                   bool stable,
-                                   bool keepdims) const {
+                                   bool stable) const {
     if (length() == 0) {
-      return shallow_copy();
+      return std::make_shared<NumpyArray>(Index64(0));
     }
     ContentPtr simplified = simplify_uniontype(true, true);
     if (dynamic_cast<UnionArray8_32*>(simplified.get())  ||
@@ -2131,11 +2134,11 @@ namespace awkward {
     }
     return simplified.get()->argsort_next(negaxis,
                                           starts,
+                                          shifts,
                                           parents,
                                           outlength,
                                           ascending,
-                                          stable,
-                                          keepdims);
+                                          stable);
   }
 
   template <typename T, typename I>
@@ -2214,36 +2217,6 @@ namespace awkward {
                                                       tail);
   }
 
-  template <typename T, typename I>
-  const ContentPtr
-  UnionArrayOf<T, I>::getitem_next_jagged(const Index64& slicestarts,
-                                          const Index64& slicestops,
-                                          const SliceVarNewAxis& slicecontent,
-                                          const Slice& tail) const {
-    return getitem_next_jagged_generic<SliceVarNewAxis>(slicestarts,
-                                                        slicestops,
-                                                        slicecontent,
-                                                        tail);
-  }
-
-  template <typename T, typename I>
-  const ContentPtr
-  UnionArrayOf<T, I>::getitem_next(const SliceVarNewAxis& varnewaxis,
-                                   const Slice& tail,
-                                   const Index64& advanced) const {
-    throw std::runtime_error(
-      std::string("FIXME: operation not yet implemented: UnionArrayOf<T, I>::getitem_next")
-      + FILENAME(__LINE__));
-  }
-
-  template <typename T, typename I>
-  const SliceJagged64
-  UnionArrayOf<T, I>::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    throw std::runtime_error(
-      std::string("FIXME: operation not yet implemented: UnionArrayOf<T, I>::varaxis_to_jagged")
-      + FILENAME(__LINE__));
-  }
-
   template <typename T, typename I>
   const ContentPtr
   UnionArrayOf<T, I>::copy_to(kernel::lib ptr_lib) const {
