@@ -625,10 +625,6 @@ namespace awkward {
              dynamic_cast<SliceMissing64*>(head.get())) {
       return Content::getitem_next(*missing, tail, advanced);
     }
-    else if (SliceVarNewAxis* varnewaxis =
-             dynamic_cast<SliceVarNewAxis*>(head.get())) {
-      return UnmaskedArray::getitem_next(*varnewaxis, tail, advanced);
-    }
     else {
       throw std::runtime_error(
         std::string("unrecognized slice type") + FILENAME(__LINE__));
@@ -700,7 +696,20 @@ namespace awkward {
     if (paramcheck != std::string("")) {
       return paramcheck;
     }
-    return content_.get()->validityerror(path + std::string(".content"));
+    if (dynamic_cast<BitMaskedArray*>(content_.get())  ||
+        dynamic_cast<ByteMaskedArray*>(content_.get())  ||
+        dynamic_cast<IndexedArray32*>(content_.get())  ||
+        dynamic_cast<IndexedArrayU32*>(content_.get())  ||
+        dynamic_cast<IndexedArray64*>(content_.get())  ||
+        dynamic_cast<IndexedOptionArray32*>(content_.get())  ||
+        dynamic_cast<IndexedOptionArray64*>(content_.get())  ||
+        dynamic_cast<UnmaskedArray*>(content_.get())) {
+      return classname() + " contains " + content_.get()->classname() +
+             ", the operation that made it might have forgotten to call 'simplify_optiontype()'";
+    }
+    else {
+      return content_.get()->validityerror(path + std::string(".content"));
+    }
   }
 
   const ContentPtr
@@ -955,8 +964,7 @@ namespace awkward {
                            const Index64& parents,
                            int64_t outlength,
                            bool ascending,
-                           bool stable,
-                           bool keepdims) const {
+                           bool stable) const {
     if (length() == 0) {
       return shallow_copy();
     }
@@ -965,17 +973,15 @@ namespace awkward {
                                                              parents,
                                                              outlength,
                                                              ascending,
-                                                             stable,
-                                                             keepdims);
+                                                             stable);
     if (RegularArray* raw = dynamic_cast<RegularArray*>(out.get())) {
-      std::shared_ptr<Content> wrapped = std::make_shared<UnmaskedArray>(
-          Identities::none(),
-          parameters_,
-          raw->content());
+      UnmaskedArray tmp(Identities::none(),
+                        parameters_,
+                        raw->content());
       return std::make_shared<RegularArray>(
           raw->identities(),
           raw->parameters(),
-          wrapped,
+          tmp.simplify_optiontype(),
           raw->size(),
           length());
     }
@@ -987,30 +993,29 @@ namespace awkward {
   const ContentPtr
   UnmaskedArray::argsort_next(int64_t negaxis,
                               const Index64& starts,
+                              const Index64& shifts,
                               const Index64& parents,
                               int64_t outlength,
                               bool ascending,
-                              bool stable,
-                              bool keepdims) const {
+                              bool stable) const {
     if (length() == 0) {
-      return shallow_copy();
+      return std::make_shared<NumpyArray>(Index64(0));
     }
     std::shared_ptr<Content> out = content_.get()->argsort_next(negaxis,
                                                                 starts,
+                                                                shifts,
                                                                 parents,
                                                                 outlength,
                                                                 ascending,
-                                                                stable,
-                                                                keepdims);
+                                                                stable);
     if (RegularArray* raw = dynamic_cast<RegularArray*>(out.get())) {
-      std::shared_ptr<Content> wrapped = std::make_shared<UnmaskedArray>(
-          Identities::none(),
-          parameters_,
-          raw->content());
+      UnmaskedArray tmp(Identities::none(),
+                        parameters_,
+                        raw->content());
       return std::make_shared<RegularArray>(
           raw->identities(),
           raw->parameters(),
-          wrapped,
+          tmp.simplify_optiontype(),
           raw->size(),
           length());
     }
@@ -1088,30 +1093,6 @@ namespace awkward {
                                                       tail);
   }
 
-  const ContentPtr
-  UnmaskedArray::getitem_next_jagged(const Index64& slicestarts,
-                                     const Index64& slicestops,
-                                     const SliceVarNewAxis& slicecontent,
-                                     const Slice& tail) const {
-    return getitem_next_jagged_generic<SliceVarNewAxis>(slicestarts,
-                                                        slicestops,
-                                                        slicecontent,
-                                                        tail);
-  }
-
-  const ContentPtr
-  UnmaskedArray::getitem_next(const SliceVarNewAxis& varnewaxis,
-                              const Slice& tail,
-                              const Index64& advanced) const {
-    SliceJagged64 jagged = content_.get()->varaxis_to_jagged(varnewaxis);
-    return getitem_next(jagged, tail, advanced);
-  }
-
-  const SliceJagged64
-  UnmaskedArray::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    return content_.get()->varaxis_to_jagged(varnewaxis);
-  }
-
   const ContentPtr
   UnmaskedArray::copy_to(kernel::lib ptr_lib) const {
     ContentPtr content = content_.get()->copy_to(ptr_lib);
