@@ -885,32 +885,30 @@ namespace awkward {
                           const Index64& parents,
                           int64_t outlength,
                           bool ascending,
-                          bool stable,
-                          bool keepdims) const {
+                          bool stable) const {
     return array().get()->sort_next(negaxis,
                                     starts,
                                     parents,
                                     outlength,
                                     ascending,
-                                    stable,
-                                    keepdims);
+                                    stable);
   }
 
   const ContentPtr
   VirtualArray::argsort_next(int64_t negaxis,
                              const Index64& starts,
+                             const Index64& shifts,
                              const Index64& parents,
                              int64_t outlength,
                              bool ascending,
-                             bool stable,
-                             bool keepdims) const {
+                             bool stable) const {
     return array().get()->argsort_next(negaxis,
                                        starts,
+                                       shifts,
                                        parents,
                                        outlength,
                                        ascending,
-                                       stable,
-                                       keepdims);
+                                       stable);
   }
 
   const ContentPtr
@@ -1163,29 +1161,6 @@ namespace awkward {
                                               tail);
   }
 
-  const ContentPtr
-  VirtualArray::getitem_next_jagged(const Index64& slicestarts,
-                                    const Index64& slicestops,
-                                    const SliceVarNewAxis& slicecontent,
-                                    const Slice& tail) const {
-    return array().get()->getitem_next_jagged(slicestarts,
-                                              slicestops,
-                                              slicecontent,
-                                              tail);
-  }
-
-  const ContentPtr
-  VirtualArray::getitem_next(const SliceVarNewAxis& varnewaxis,
-                             const Slice& tail,
-                             const Index64& advanced) const {
-    return array().get()->getitem_next(varnewaxis, tail, advanced);
-  }
-
-  const SliceJagged64
-  VirtualArray::varaxis_to_jagged(const SliceVarNewAxis& varnewaxis) const {
-    return array().get()->varaxis_to_jagged(varnewaxis);
-  }
-
   const ContentPtr
   VirtualArray::copy_to(kernel::lib ptr_lib) const {
     IdentitiesPtr identities(nullptr);
