@@ -2,9 +2,8 @@
 
 #define FILENAME(line) FILENAME_FOR_EXCEPTIONS("src/libawkward/builder/BoolBuilder.cpp", line)
 
-#include "awkward/Identities.h"
-#include "awkward/array/NumpyArray.h"
-#include "awkward/type/PrimitiveType.h"
+#include <stdexcept>
+
 #include "awkward/builder/ArrayBuilderOptions.h"
 #include "awkward/builder/OptionBuilder.h"
 #include "awkward/builder/UnionBuilder.h"
@@ -38,22 +37,6 @@ namespace awkward {
     buffer_.clear();
   }
 
-  const ContentPtr
-  BoolBuilder::snapshot() const {
-    std::vector<ssize_t> shape = { (ssize_t)buffer_.length() };
-    std::vector<ssize_t> strides = { (ssize_t)sizeof(bool) };
-    return std::make_shared<NumpyArray>(Identities::none(),
-                                        util::Parameters(),
-                                        buffer_.ptr(),
-                                        shape,
-                                        strides,
-                                        0,
-                                        sizeof(bool),
-                                        "?",
-                                        util::dtype::boolean,
-                                        kernel::lib::cpu);
-  }
-
   bool
   BoolBuilder::active() const {
     return false;
@@ -93,6 +76,20 @@ namespace awkward {
     return out;
   }
 
+  const BuilderPtr
+  BoolBuilder::datetime(int64_t x, const std::string& unit) {
+    BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+    out.get()->datetime(x, unit);
+    return out;
+  }
+
+  const BuilderPtr
+  BoolBuilder::timedelta(int64_t x, const std::string& unit) {
+    BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+    out.get()->timedelta(x, unit);
+    return out;
+  }
+
   const BuilderPtr
   BoolBuilder::string(const char* x, int64_t length, const char* encoding) {
     BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
