@@ -2,9 +2,8 @@
 
 #define FILENAME(line) FILENAME_FOR_EXCEPTIONS("src/libawkward/builder/Int64Builder.cpp", line)
 
-#include "awkward/Identities.h"
-#include "awkward/array/NumpyArray.h"
-#include "awkward/type/PrimitiveType.h"
+#include <stdexcept>
+
 #include "awkward/builder/ArrayBuilderOptions.h"
 #include "awkward/builder/Complex128Builder.h"
 #include "awkward/builder/Float64Builder.h"
@@ -45,23 +44,6 @@ namespace awkward {
     buffer_.clear();
   }
 
-  const ContentPtr
-  Int64Builder::snapshot() const {
-    std::vector<ssize_t> shape = { (ssize_t)buffer_.length() };
-    std::vector<ssize_t> strides = { (ssize_t)sizeof(int64_t) };
-    return std::make_shared<NumpyArray>(
-             Identities::none(),
-             util::Parameters(),
-             buffer_.ptr(),
-             shape,
-             strides,
-             0,
-             sizeof(int64_t),
-             util::dtype_to_format(util::dtype::int64),
-             util::dtype::int64,
-             kernel::lib::cpu);
-  }
-
   bool
   Int64Builder::active() const {
     return false;
@@ -101,6 +83,20 @@ namespace awkward {
     return out;
   }
 
+  const BuilderPtr
+  Int64Builder::datetime(int64_t x, const std::string& unit) {
+    BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+    out.get()->datetime(x, unit);
+    return out;
+  }
+
+  const BuilderPtr
+  Int64Builder::timedelta(int64_t x, const std::string& unit) {
+    BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+    out.get()->timedelta(x, unit);
+    return out;
+  }
+
   const BuilderPtr
   Int64Builder::string(const char* x, int64_t length, const char* encoding) {
     BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
