@@ -4,10 +4,6 @@
 
 #include <stdexcept>
 
-#include "awkward/Identities.h"
-#include "awkward/Index.h"
-#include "awkward/array/ListOffsetArray.h"
-#include "awkward/type/ListType.h"
 #include "awkward/builder/ArrayBuilderOptions.h"
 #include "awkward/builder/OptionBuilder.h"
 #include "awkward/builder/UnionBuilder.h"
@@ -52,15 +48,6 @@ namespace awkward {
     content_.get()->clear();
   }
 
-  const ContentPtr
-  ListBuilder::snapshot() const {
-    Index64 offsets(offsets_.ptr(), 0, offsets_.length(), kernel::lib::cpu);
-    return std::make_shared<ListOffsetArray64>(Identities::none(),
-                                               util::Parameters(),
-                                               offsets,
-                                               content_.get()->snapshot());
-  }
-
   bool
   ListBuilder::active() const {
     return begun_;
@@ -131,6 +118,32 @@ namespace awkward {
     }
   }
 
+  const BuilderPtr
+  ListBuilder::datetime(int64_t x, const std::string& unit) {
+    if (!begun_) {
+      BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+      out.get()->datetime(x, unit);
+      return out;
+    }
+    else {
+      maybeupdate(content_.get()->datetime(x, unit));
+      return shared_from_this();
+    }
+  }
+
+  const BuilderPtr
+  ListBuilder::timedelta(int64_t x, const std::string& unit) {
+    if (!begun_) {
+      BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+      out.get()->timedelta(x, unit);
+      return out;
+    }
+    else {
+      maybeupdate(content_.get()->timedelta(x, unit));
+      return shared_from_this();
+    }
+  }
+
   const BuilderPtr
   ListBuilder::string(const char* x, int64_t length, const char* encoding) {
     if (!begun_) {
