@@ -4,10 +4,6 @@
 
 #include <stdexcept>
 
-#include "awkward/Identities.h"
-#include "awkward/Index.h"
-#include "awkward/array/IndexedArray.h"
-#include "awkward/type/OptionType.h"
 #include "awkward/builder/ArrayBuilderOptions.h"
 
 #include "awkward/builder/OptionBuilder.h"
@@ -58,15 +54,6 @@ namespace awkward {
     content_.get()->clear();
   }
 
-  const ContentPtr
-  OptionBuilder::snapshot() const {
-    Index64 index(index_.ptr(), 0, index_.length(), kernel::lib::cpu);
-    return IndexedOptionArray64(Identities::none(),
-                                util::Parameters(),
-                                index,
-                                content_.get()->snapshot()).simplify_optiontype();
-  }
-
   bool
   OptionBuilder::active() const {
     return content_.get()->active();
@@ -135,6 +122,32 @@ namespace awkward {
     return shared_from_this();
   }
 
+  const BuilderPtr
+  OptionBuilder::datetime(int64_t x, const std::string& unit) {
+    if (!content_.get()->active()) {
+      int64_t length = content_.get()->length();
+      maybeupdate(content_.get()->datetime(x, unit));
+      index_.append(length);
+    }
+    else {
+      content_.get()->datetime(x, unit);
+    }
+    return shared_from_this();
+  }
+
+  const BuilderPtr
+  OptionBuilder::timedelta(int64_t x, const std::string& unit) {
+    if (!content_.get()->active()) {
+      int64_t length = content_.get()->length();
+      maybeupdate(content_.get()->timedelta(x, unit));
+      index_.append(length);
+    }
+    else {
+      content_.get()->timedelta(x, unit);
+    }
+    return shared_from_this();
+  }
+
   const BuilderPtr
   OptionBuilder::string(const char* x, int64_t length, const char* encoding) {
     if (!content_.get()->active()) {
