@@ -4,22 +4,13 @@
 
 #include <stdexcept>
 
-#include "awkward/Identities.h"
-#include "awkward/Index.h"
-
-#include "awkward/Identities.h"
-#include "awkward/Index.h"
-#include "awkward/array/RecordArray.h"
-#include "awkward/array/EmptyArray.h"
-#include "awkward/type/RecordType.h"
-#include "awkward/type/UnknownType.h"
 #include "awkward/builder/ArrayBuilderOptions.h"
 #include "awkward/builder/OptionBuilder.h"
 #include "awkward/builder/UnionBuilder.h"
 #include "awkward/builder/UnknownBuilder.h"
-#include "awkward/virtual/ArrayCache.h"
 
 #include "awkward/builder/RecordBuilder.h"
+#include "awkward/util.h"
 
 namespace awkward {
   const BuilderPtr
@@ -55,7 +46,8 @@ namespace awkward {
       , length_(length)
       , begun_(begun)
       , nextindex_(nextindex)
-      , nexttotry_(nexttotry) { }
+      , nexttotry_(nexttotry)
+      , keys_size_((int64_t)keys.size()) { }
 
   const std::string
   RecordBuilder::name() const {
@@ -90,32 +82,7 @@ namespace awkward {
     begun_ = false;
     nextindex_ = -1;
     nexttotry_ = 0;
-  }
-
-  const ContentPtr
-  RecordBuilder::snapshot() const {
-    if (length_ == -1) {
-      return std::make_shared<EmptyArray>(Identities::none(),
-                                          util::Parameters());
-    }
-    util::Parameters parameters;
-    if (nameptr_ != nullptr) {
-      parameters["__record__"] = util::quote(name_);
-    }
-    ContentPtrVec contents;
-    util::RecordLookupPtr recordlookup =
-      std::make_shared<util::RecordLookup>();
-    for (size_t i = 0;  i < contents_.size();  i++) {
-      contents.push_back(contents_[i].get()->snapshot());
-      recordlookup.get()->push_back(keys_[i]);
-    }
-    std::vector<ArrayCachePtr> caches;  // nothing is virtual here
-    return std::make_shared<RecordArray>(Identities::none(),
-                                         parameters,
-                                         contents,
-                                         recordlookup,
-                                         length_,
-                                         caches);
+    keys_size_ = 0;
   }
 
   bool
@@ -228,6 +195,48 @@ namespace awkward {
     return shared_from_this();
   }
 
+  const BuilderPtr
+  RecordBuilder::datetime(int64_t x, const std::string& unit) {
+    if (!begun_) {
+      BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+      out.get()->datetime(x, unit);
+      return out;
+    }
+    else if (nextindex_ == -1) {
+      throw std::invalid_argument(
+        std::string("called 'datetime' immediately after 'begin_record'; "
+                    "needs 'index' or 'end_record'") + FILENAME(__LINE__));
+    }
+    else if (!contents_[(size_t)nextindex_].get()->active()) {
+      maybeupdate(nextindex_, contents_[(size_t)nextindex_].get()->datetime(x, unit));
+    }
+    else {
+      contents_[(size_t)nextindex_].get()->datetime(x, unit);
+    }
+    return shared_from_this();
+  }
+
+  const BuilderPtr
+  RecordBuilder::timedelta(int64_t x, const std::string& unit) {
+    if (!begun_) {
+      BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+      out.get()->timedelta(x, unit);
+      return out;
+    }
+    else if (nextindex_ == -1) {
+      throw std::invalid_argument(
+        std::string("called 'timedelta' immediately after 'begin_record'; "
+                    "needs 'index' or 'end_record'") + FILENAME(__LINE__));
+    }
+    else if (!contents_[(size_t)nextindex_].get()->active()) {
+      maybeupdate(nextindex_, contents_[(size_t)nextindex_].get()->timedelta(x, unit));
+    }
+    else {
+      contents_[(size_t)nextindex_].get()->timedelta(x, unit);
+    }
+    return shared_from_this();
+  }
+
   const BuilderPtr
   RecordBuilder::string(const char* x, int64_t length, const char* encoding) {
     if (!begun_) {
@@ -414,10 +423,9 @@ namespace awkward {
     }
     else if (nextindex_ == -1  ||
              !contents_[(size_t)nextindex_].get()->active()) {
-      int64_t wrap_around = (int64_t)pointers_.size();
       int64_t i = nexttotry_;
       do {
-        if (i >= wrap_around) {
+        if (i >= keys_size_) {
           i = 0;
           if (i == nexttotry_) {
             break;
@@ -430,7 +438,7 @@ namespace awkward {
         }
         i++;
       } while (i != nexttotry_);
-      nextindex_ = wrap_around;
+      nextindex_ = keys_size_;
       nexttotry_ = 0;
       if (length_ == 0) {
         contents_.push_back(UnknownBuilder::fromempty(options_));
@@ -443,6 +451,7 @@ namespace awkward {
       }
       keys_.push_back(std::string(key));
       pointers_.push_back(key);
+      keys_size_ = (int64_t)keys_.size();
       return shared_from_this();
     }
     else {
@@ -460,10 +469,9 @@ namespace awkward {
     }
     else if (nextindex_ == -1  ||
              !contents_[(size_t)nextindex_].get()->active()) {
-      int64_t wrap_around = (int64_t)keys_.size();
       int64_t i = nexttotry_;
       do {
-        if (i >= wrap_around) {
+        if (i >= keys_size_) {
           i = 0;
           if (i == nexttotry_) {
             break;
@@ -476,7 +484,7 @@ namespace awkward {
         }
         i++;
       } while (i != nexttotry_);
-      nextindex_ = wrap_around;
+      nextindex_ = keys_size_;
       nexttotry_ = 0;
       if (length_ == 0) {
         contents_.push_back(UnknownBuilder::fromempty(options_));
@@ -489,6 +497,7 @@ namespace awkward {
       }
       keys_.push_back(std::string(key));
       pointers_.push_back(nullptr);
+      keys_size_ = (int64_t)keys_.size();
       return shared_from_this();
     }
     else {
