@@ -4,12 +4,6 @@
 
 #include <stdexcept>
 
-#include "awkward/Identities.h"
-#include "awkward/Index.h"
-#include "awkward/array/RecordArray.h"
-#include "awkward/array/EmptyArray.h"
-#include "awkward/type/RecordType.h"
-#include "awkward/type/UnknownType.h"
 #include "awkward/builder/ArrayBuilderOptions.h"
 #include "awkward/builder/OptionBuilder.h"
 #include "awkward/builder/UnionBuilder.h"
@@ -63,25 +57,6 @@ namespace awkward {
     nextindex_ = -1;
   }
 
-  const ContentPtr
-  TupleBuilder::snapshot() const {
-    if (length_ == -1) {
-      return std::make_shared<EmptyArray>(Identities::none(),
-                                          util::Parameters());
-    }
-    ContentPtrVec contents;
-    for (size_t i = 0;  i < contents_.size();  i++) {
-      contents.push_back(contents_[i].get()->snapshot());
-    }
-    std::vector<ArrayCachePtr> caches;  // nothing is virtual here
-    return std::make_shared<RecordArray>(Identities::none(),
-                                         util::Parameters(),
-                                         contents,
-                                         util::RecordLookupPtr(nullptr),
-                                         length_,
-                                         caches);
-  }
-
   bool
   TupleBuilder::active() const {
     return begun_;
@@ -192,6 +167,48 @@ namespace awkward {
     return shared_from_this();
   }
 
+  const BuilderPtr
+  TupleBuilder::datetime(int64_t x, const std::string& unit) {
+    if (!begun_) {
+      BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+      out.get()->datetime(x, unit);
+      return out;
+    }
+    else if (nextindex_ == -1) {
+      throw std::invalid_argument(
+        std::string("called 'datetime' immediately after 'begin_tuple'; "
+                    "needs 'index' or 'end_tuple'") + FILENAME(__LINE__));
+    }
+    else if (!contents_[(size_t)nextindex_].get()->active()) {
+      maybeupdate(nextindex_, contents_[(size_t)nextindex_].get()->datetime(x, unit));
+    }
+    else {
+      contents_[(size_t)nextindex_].get()->datetime(x, unit);
+    }
+    return shared_from_this();
+  }
+
+  const BuilderPtr
+  TupleBuilder::timedelta(int64_t x, const std::string& unit) {
+    if (!begun_) {
+      BuilderPtr out = UnionBuilder::fromsingle(options_, shared_from_this());
+      out.get()->timedelta(x, unit);
+      return out;
+    }
+    else if (nextindex_ == -1) {
+      throw std::invalid_argument(
+        std::string("called 'timedelta' immediately after 'begin_tuple'; "
+                    "needs 'index' or 'end_tuple'") + FILENAME(__LINE__));
+    }
+    else if (!contents_[(size_t)nextindex_].get()->active()) {
+      maybeupdate(nextindex_, contents_[(size_t)nextindex_].get()->timedelta(x, unit));
+    }
+    else {
+      contents_[(size_t)nextindex_].get()->timedelta(x, unit);
+    }
+    return shared_from_this();
+  }
+
   const BuilderPtr
   TupleBuilder::string(const char* x, int64_t length, const char* encoding) {
     if (!begun_) {
