@@ -4,12 +4,10 @@
 
 #include <stdexcept>
 
-#include "awkward/Identities.h"
-#include "awkward/Index.h"
-#include "awkward/type/UnionType.h"
 #include "awkward/builder/ArrayBuilderOptions.h"
 #include "awkward/builder/OptionBuilder.h"
 #include "awkward/builder/BoolBuilder.h"
+#include "awkward/builder/DatetimeBuilder.h"
 #include "awkward/builder/Int64Builder.h"
 #include "awkward/builder/Float64Builder.h"
 #include "awkward/builder/StringBuilder.h"
@@ -18,7 +16,6 @@
 #include "awkward/builder/RecordBuilder.h"
 #include "awkward/builder/IndexedBuilder.h"
 #include "awkward/builder/Complex128Builder.h"
-#include "awkward/array/UnionArray.h"
 
 #include "awkward/builder/UnionBuilder.h"
 
@@ -66,21 +63,6 @@ namespace awkward {
     }
   }
 
-  const ContentPtr
-  UnionBuilder::snapshot() const {
-    Index8 tags(tags_.ptr(), 0, tags_.length(), kernel::lib::cpu);
-    Index64 index(index_.ptr(), 0, index_.length(), kernel::lib::cpu);
-    ContentPtrVec contents;
-    for (auto content : contents_) {
-      contents.push_back(content.get()->snapshot());
-    }
-    return UnionArray8_64(Identities::none(),
-                          util::Parameters(),
-                          tags,
-                          index,
-                          contents).simplify_uniontype(true, false);
-  }
-
   bool
   UnionBuilder::active() const {
     return current_ != -1;
@@ -239,10 +221,10 @@ namespace awkward {
             dynamic_cast<Int64Builder*>(tofill.get())->buffer());
           contents_[(size_t)i] = tofill;
         }
-      }
-      else {
-        tofill = Complex128Builder::fromempty(options_);
-        contents_.push_back(tofill);
+        else {
+          tofill = Complex128Builder::fromempty(options_);
+          contents_.push_back(tofill);
+        }
       }
       int64_t length = tofill.get()->length();
       tofill.get()->complex(x);
@@ -255,6 +237,64 @@ namespace awkward {
     return shared_from_this();
   }
 
+  const BuilderPtr
+  UnionBuilder::datetime(int64_t x, const std::string& unit) {
+    if (current_ == -1) {
+      BuilderPtr tofill(nullptr);
+      int8_t i = 0;
+      for (auto content : contents_) {
+        if (DatetimeBuilder* raw = dynamic_cast<DatetimeBuilder*>(content.get())) {
+          if (raw->units() == unit) {
+            tofill = content;
+            break;
+          }
+        }
+        i++;
+      }
+      if (tofill.get() == nullptr) {
+        tofill = DatetimeBuilder::fromempty(options_, unit);
+        contents_.push_back(tofill);
+      }
+      int64_t len = tofill.get()->length();
+      tofill.get()->datetime(x, unit);
+      tags_.append(i);
+      index_.append(len);
+    }
+    else {
+      contents_[(size_t)current_].get()->datetime(x, unit);
+    }
+    return shared_from_this();
+  }
+
+  const BuilderPtr
+  UnionBuilder::timedelta(int64_t x, const std::string& unit) {
+    if (current_ == -1) {
+      BuilderPtr tofill(nullptr);
+      int8_t i = 0;
+      for (auto content : contents_) {
+        if (DatetimeBuilder* raw = dynamic_cast<DatetimeBuilder*>(content.get())) {
+          if (raw->units() == unit) {
+            tofill = content;
+            break;
+          }
+        }
+        i++;
+      }
+      if (tofill.get() == nullptr) {
+        tofill = DatetimeBuilder::fromempty(options_, unit);
+        contents_.push_back(tofill);
+      }
+      int64_t len = tofill.get()->length();
+      tofill.get()->timedelta(x, unit);
+      tags_.append(i);
+      index_.append(len);
+    }
+    else {
+      contents_[(size_t)current_].get()->timedelta(x, unit);
+    }
+    return shared_from_this();
+  }
+
   const BuilderPtr
   UnionBuilder::string(const char* x, int64_t length, const char* encoding) {
     if (current_ == -1) {
@@ -455,42 +495,42 @@ namespace awkward {
       int8_t i = 0;
       for (auto content : contents_) {
         if (IndexedGenericBuilder* raw =
-            dynamic_cast<IndexedGenericBuilder*>(content.get())) {
+          dynamic_cast<IndexedGenericBuilder*>(content.get())) {
           if (raw->arrayptr() == array.get()) {
             tofill = content;
             break;
           }
         }
         else if (IndexedI32Builder* raw =
-                 dynamic_cast<IndexedI32Builder*>(content.get())) {
+          dynamic_cast<IndexedI32Builder*>(content.get())) {
           if (raw->arrayptr() == array.get()) {
             tofill = content;
             break;
           }
         }
         else if (IndexedIU32Builder* raw =
-                 dynamic_cast<IndexedIU32Builder*>(content.get())) {
+          dynamic_cast<IndexedIU32Builder*>(content.get())) {
           if (raw->arrayptr() == array.get()) {
             tofill = content;
             break;
           }
         }
         else if (IndexedI64Builder* raw =
-                 dynamic_cast<IndexedI64Builder*>(content.get())) {
+          dynamic_cast<IndexedI64Builder*>(content.get())) {
           if (raw->arrayptr() == array.get()) {
             tofill = content;
             break;
           }
         }
         else if (IndexedIO32Builder* raw =
-                 dynamic_cast<IndexedIO32Builder*>(content.get())) {
+          dynamic_cast<IndexedIO32Builder*>(content.get())) {
           if (raw->arrayptr() == array.get()) {
             tofill = content;
             break;
           }
         }
         else if (IndexedIO64Builder* raw =
-                 dynamic_cast<IndexedIO64Builder*>(content.get())) {
+          dynamic_cast<IndexedIO64Builder*>(content.get())) {
           if (raw->arrayptr() == array.get()) {
             tofill = content;
             break;
