@@ -4,14 +4,10 @@
 
 #include <stdexcept>
 
-#include "awkward/Identities.h"
-#include "awkward/Index.h"
-#include "awkward/array/EmptyArray.h"
-#include "awkward/array/IndexedArray.h"
-#include "awkward/type/UnknownType.h"
 #include "awkward/builder/ArrayBuilderOptions.h"
 #include "awkward/builder/OptionBuilder.h"
 #include "awkward/builder/BoolBuilder.h"
+#include "awkward/builder/DatetimeBuilder.h"
 #include "awkward/builder/Int64Builder.h"
 #include "awkward/builder/Float64Builder.h"
 #include "awkward/builder/Complex128Builder.h"
@@ -49,28 +45,6 @@ namespace awkward {
     nullcount_ = 0;
   }
 
-  const ContentPtr
-  UnknownBuilder::snapshot() const {
-    if (nullcount_ == 0) {
-      return std::make_shared<EmptyArray>(Identities::none(),
-                                          util::Parameters());
-    }
-    else {
-      // This is the only snapshot that is O(N), rather than O(1),
-      // but it is a corner case (array of only Nones).
-      Index64 index(nullcount_);
-      int64_t* rawptr = index.ptr().get();
-      for (int64_t i = 0;  i < nullcount_;  i++) {
-        rawptr[i] = -1;
-      }
-      return std::make_shared<IndexedOptionArray64>(
-        Identities::none(),
-        util::Parameters(),
-        index,
-        std::make_shared<EmptyArray>(Identities::none(), util::Parameters()));
-    }
-  }
-
   bool
   UnknownBuilder::active() const {
     return false;
@@ -122,6 +96,26 @@ namespace awkward {
     return out;
   }
 
+  const BuilderPtr
+  UnknownBuilder::datetime(int64_t x, const std::string& unit) {
+    BuilderPtr out = DatetimeBuilder::fromempty(options_, unit);
+    if (nullcount_ != 0) {
+      out = OptionBuilder::fromnulls(options_, nullcount_, out);
+    }
+    out.get()->datetime(x, unit);
+    return out;
+  }
+
+  const BuilderPtr
+  UnknownBuilder::timedelta(int64_t x, const std::string& unit) {
+    BuilderPtr out = DatetimeBuilder::fromempty(options_, unit);
+    if (nullcount_ != 0) {
+      out = OptionBuilder::fromnulls(options_, nullcount_, out);
+    }
+    out.get()->timedelta(x, unit);
+    return out;
+  }
+
   const BuilderPtr
   UnknownBuilder::string(const char* x, int64_t length, const char* encoding) {
     BuilderPtr out = StringBuilder::fromempty(options_, encoding);
