@@ -15,6 +15,7 @@
 
 #include "awkward/builder/ArrayBuilder.h"
 #include "awkward/Content.h"
+#include "awkward/refactoring.h"
 
 #include "awkward/io/json.h"
 
@@ -46,7 +47,7 @@ namespace awkward {
       writer.Int64(value.GetInt());
     }
     else if (value.IsDouble()) {
-      writer.Int64((int64_t)value.GetDouble());
+      writer.Double(value.GetDouble());
     }
     else if (value.IsString()) {
       writer.String(value.GetString());
@@ -757,8 +758,9 @@ namespace awkward {
       return true;
     }
 
+    // FIXME: refactor
     const ContentPtr snapshot() const {
-      return builder_.snapshot();
+      return ::builder_snapshot(builder_.builder());
     }
 
   private:
