@@ -1,4 +1,4 @@
-// BSD 3-Clause License; see https://github.com/jpivarski/awkward-1.0/blob/master/LICENSE
+// BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
 
 #include <string>
 #include <sstream>
@@ -22,11 +22,11 @@ namespace awkward {
                           const std::string& post) const {
     std::string typestr;
     if (get_typestr(typestr)) {
-      return typestr;
+      return wrap_categorical(typestr);
     }
 
     std::stringstream out;
-    if (parameters_.empty()) {
+    if (parameters_empty()) {
       out << indent << pre << "var * "
           << type_.get()->tostring_part(indent, "", "") << post;
     }
@@ -35,7 +35,7 @@ namespace awkward {
           << type_.get()->tostring_part(indent, "", "") << ", "
           << string_parameters() << "]" << post;
     }
-    return out.str();
+    return wrap_categorical(out.str());
   }
 
   const TypePtr
@@ -46,7 +46,7 @@ namespace awkward {
   bool
   ListType::equal(const TypePtr& other, bool check_parameters) const {
     if (ListType* t = dynamic_cast<ListType*>(other.get())) {
-      if (check_parameters  &&  !parameters_equal(other.get()->parameters())) {
+      if (check_parameters  &&  !parameters_equal(other.get()->parameters(), false)) {
         return false;
       }
       return type().get()->equal(t->type(), check_parameters);
