@@ -1,4 +1,4 @@
-// BSD 3-Clause License; see https://github.com/jpivarski/awkward-1.0/blob/master/LICENSE
+// BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
 
 #include <string>
 #include <sstream>
@@ -24,11 +24,11 @@ namespace awkward {
                              const std::string& post) const {
     std::string typestr;
     if (get_typestr(typestr)) {
-      return typestr;
+      return wrap_categorical(typestr);
     }
 
     std::stringstream out;
-    if (parameters_.empty()) {
+    if (parameters_empty()) {
       out << indent << pre << size_ << " * "
           << type_.get()->tostring_part(indent, "", "") << post;
     }
@@ -37,7 +37,7 @@ namespace awkward {
           << type_.get()->tostring_part(indent, "", "") << ", "
           << string_parameters() << "]" << post;
     }
-    return out.str();
+    return wrap_categorical(out.str());
   }
 
   const TypePtr
@@ -48,7 +48,7 @@ namespace awkward {
   bool
   RegularType::equal(const TypePtr& other, bool check_parameters) const {
     if (RegularType* t = dynamic_cast<RegularType*>(other.get())) {
-      if (check_parameters  &&  !parameters_equal(other.get()->parameters())) {
+      if (check_parameters  &&  !parameters_equal(other.get()->parameters(), false)) {
         return false;
       }
       return (size() == t->size()  &&
@@ -90,7 +90,8 @@ namespace awkward {
     return std::make_shared<RegularArray>(Identities::none(),
                                           parameters_,
                                           content,
-                                          size_);
+                                          size_,
+                                          0);
   }
 
   const TypePtr
