@@ -25,7 +25,7 @@ def test_fromfile(tmp_path):
     a = ak.from_json(os.path.join(str(tmp_path), "tmp1.json"))
     assert ak.to_list(a) == [[1.1, 2.2, 3.0], [], [4.0, 5.5]]
 
-    with pytest.raises(ValueError):
+    with pytest.raises(IOError):
         ak.from_json("nonexistent.json")
 
     with open(os.path.join(str(tmp_path), "tmp2.json"), "w") as f:
@@ -39,23 +39,15 @@ def test_tostring():
     content = ak.layout.NumpyArray(np.arange(2 * 3 * 5 * 7).reshape(-1, 7))
     offsetsA = np.arange(0, 2 * 3 * 5 + 5, 5)
     offsetsB = np.arange(0, 2 * 3 + 3, 3)
-    startsA, stopsA = offsetsA[:-1], offsetsA[1:]
-    startsB, stopsB = offsetsB[:-1], offsetsB[1:]
 
     listoffsetarrayA32 = ak.layout.ListOffsetArray32(
         ak.layout.Index32(offsetsA), content
     )
-    listarrayA32 = ak.layout.ListArray32(
-        ak.layout.Index32(startsA), ak.layout.Index32(stopsA), content
-    )
     modelA = np.arange(2 * 3 * 5 * 7).reshape(2 * 3, 5, 7)
 
     listoffsetarrayB32 = ak.layout.ListOffsetArray32(
         ak.layout.Index32(offsetsB), listoffsetarrayA32
     )
-    listarrayB32 = ak.layout.ListArray32(
-        ak.layout.Index32(startsB), ak.layout.Index32(stopsB), listarrayA32
-    )
     modelB = np.arange(2 * 3 * 5 * 7).reshape(2, 3, 5, 7)
 
     assert content.tojson() == json.dumps(ak.to_list(content), separators=(",", ":"))
