@@ -9,7 +9,7 @@ import numpy as np  # noqa: F401
 import awkward as ak  # noqa: F401
 
 
-pyarrow = pytest.importorskip("pyarrow")
+pytest.importorskip("pyarrow.parquet")
 
 
 @pytest.mark.parametrize("one,two,three", [(1, 2, 3), ("one", "two", "three")])
@@ -125,12 +125,12 @@ def test_7(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y:x[0]"])
     assert np.asarray(array.layout.field("x").array.offsets).tolist() == [0, 0, 1, 4]
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y:x[0]"])
     array.layout.field("x").array.content.field("y").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.y[0]", "tmp:col:x.list.item.y[0]"]
+        ["tmp:off:x.list.item.y:x[0]", "tmp:col:x.list.item.y[0]"]
     )
     assert array.tolist() == data
 
@@ -147,17 +147,17 @@ def test_8(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y:x[0]"])
     assert np.asarray(array.layout.field("x").array.offsets).tolist() == [0, 0, 1, 4]
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y:x[0]"])
     array.layout.field("x").array.content.field("y").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.y[0]", "tmp:col:x.list.item.y[0]"]
+        ["tmp:off:x.list.item.y:x[0]", "tmp:col:x.list.item.y[0]"]
     )
     array.layout.field("x").array.content.field("z").array
     assert set(array.caches[0].keys()) == set(
         [
-            "tmp:off:x.list.item.y[0]",
+            "tmp:off:x.list.item.y:x[0]",
             "tmp:col:x.list.item.y[0]",
             "tmp:col:x.list.item.z[0]",
         ]
@@ -166,17 +166,17 @@ def test_8(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y:x[0]"])
     assert np.asarray(array.layout.field("x").array.offsets).tolist() == [0, 0, 1, 4]
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y:x[0]"])
     array.layout.field("x").array.content.field("z").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.y[0]", "tmp:col:x.list.item.z[0]"]
+        ["tmp:off:x.list.item.y:x[0]", "tmp:col:x.list.item.z[0]"]
     )
     array.layout.field("x").array.content.field("y").array
     assert set(array.caches[0].keys()) == set(
         [
-            "tmp:off:x.list.item.y[0]",
+            "tmp:off:x.list.item.y:x[0]",
             "tmp:col:x.list.item.z[0]",
             "tmp:col:x.list.item.y[0]",
         ]
@@ -196,14 +196,14 @@ def test_9(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     assert np.asarray(array.layout.field("x").array.offsets).tolist() == [0, 0, 1, 4]
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     array.layout.field("x").array.content.field("y").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     array.layout.field("x").array.content.field("y").array.field("q").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.y.q[0]", "tmp:col:x.list.item.y.q[0]"]
+        ["tmp:off:x.list.item.y.q:x[0]", "tmp:col:x.list.item.y.q[0]"]
     )
     assert array.tolist() == data
 
@@ -226,19 +226,19 @@ def test_10(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     assert np.asarray(array.layout.field("x").array.offsets).tolist() == [0, 0, 1, 4]
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     array.layout.field("x").array.content.field("y").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     array.layout.field("x").array.content.field("y").array.field("q").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.y.q[0]", "tmp:col:x.list.item.y.q[0]"]
+        ["tmp:off:x.list.item.y.q:x[0]", "tmp:col:x.list.item.y.q[0]"]
     )
     array.layout.field("x").array.content.field("z").array
     assert set(array.caches[0].keys()) == set(
         [
-            "tmp:off:x.list.item.y.q[0]",
+            "tmp:off:x.list.item.y.q:x[0]",
             "tmp:col:x.list.item.y.q[0]",
             "tmp:col:x.list.item.z[0]",
         ]
@@ -247,19 +247,19 @@ def test_10(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     assert np.asarray(array.layout.field("x").array.offsets).tolist() == [0, 0, 1, 4]
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     array.layout.field("x").array.content.field("y").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.q:x[0]"])
     array.layout.field("x").array.content.field("z").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.y.q[0]", "tmp:col:x.list.item.z[0]"]
+        ["tmp:off:x.list.item.y.q:x[0]", "tmp:col:x.list.item.z[0]"]
     )
     array.layout.field("x").array.content.field("y").array.field("q").array
     assert set(array.caches[0].keys()) == set(
         [
-            "tmp:off:x.list.item.y.q[0]",
+            "tmp:off:x.list.item.y.q:x[0]",
             "tmp:col:x.list.item.y.q[0]",
             "tmp:col:x.list.item.z[0]",
         ]
@@ -285,19 +285,19 @@ def test_11(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z:x[0]"])
     assert np.asarray(array.layout.field("x").array.offsets).tolist() == [0, 0, 1, 4]
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z:x[0]"])
     array.layout.field("x").array.content.field("y").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z:x[0]"])
     array.layout.field("x").array.content.field("y").array.field("q").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.z[0]", "tmp:col:x.list.item.y.q[0]"]
+        ["tmp:off:x.list.item.z:x[0]", "tmp:col:x.list.item.y.q[0]"]
     )
     array.layout.field("x").array.content.field("z").array
     assert set(array.caches[0].keys()) == set(
         [
-            "tmp:off:x.list.item.z[0]",
+            "tmp:off:x.list.item.z:x[0]",
             "tmp:col:x.list.item.y.q[0]",
             "tmp:col:x.list.item.z[0]",
         ]
@@ -306,19 +306,19 @@ def test_11(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z:x[0]"])
     assert np.asarray(array.layout.field("x").array.offsets).tolist() == [0, 0, 1, 4]
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z:x[0]"])
     array.layout.field("x").array.content.field("y").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.z:x[0]"])
     array.layout.field("x").array.content.field("z").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.z[0]", "tmp:col:x.list.item.z[0]"]
+        ["tmp:off:x.list.item.z:x[0]", "tmp:col:x.list.item.z[0]"]
     )
     array.layout.field("x").array.content.field("y").array.field("q").array
     assert set(array.caches[0].keys()) == set(
         [
-            "tmp:off:x.list.item.z[0]",
+            "tmp:off:x.list.item.z:x[0]",
             "tmp:col:x.list.item.y.q[0]",
             "tmp:col:x.list.item.z[0]",
         ]
@@ -385,15 +385,15 @@ def test_14(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.list.item[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.list.item:x[0]"])
     array.layout.field("x").array.content.field("z").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.y.list.item[0]", "tmp:col:x.list.item.z[0]"]
+        ["tmp:off:x.list.item.y.list.item:x[0]", "tmp:col:x.list.item.z[0]"]
     )
     array.layout.field("x").array.content.field("y").array
     assert set(array.caches[0].keys()) == set(
         [
-            "tmp:off:x.list.item.y.list.item[0]",
+            "tmp:off:x.list.item.y.list.item:x[0]",
             "tmp:col:x.list.item.z[0]",
             "tmp:lst:x.list.item.y[0]",
         ]
@@ -402,15 +402,15 @@ def test_14(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     array.layout.field("x").array
-    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.list.item[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:x.list.item.y.list.item:x[0]"])
     array.layout.field("x").array.content.field("y").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:x.list.item.y.list.item[0]", "tmp:lst:x.list.item.y[0]"]
+        ["tmp:off:x.list.item.y.list.item:x[0]", "tmp:lst:x.list.item.y[0]"]
     )
     array.layout.field("x").array.content.field("z").array
     assert set(array.caches[0].keys()) == set(
         [
-            "tmp:off:x.list.item.y.list.item[0]",
+            "tmp:off:x.list.item.y.list.item:x[0]",
             "tmp:lst:x.list.item.y[0]",
             "tmp:col:x.list.item.z[0]",
         ]
@@ -450,9 +450,9 @@ def test_17(one, two, three, tmp_path):
     array = ak.from_parquet(filename, lazy=True, lazy_cache_key="tmp")
     assert set(array.caches[0].keys()) == set()
     assert np.asarray(array.layout.array.offsets).tolist() == [0, 2, 2, 3]
-    assert set(array.caches[0].keys()) == set(["tmp:off:.list.item.x[0]"])
+    assert set(array.caches[0].keys()) == set(["tmp:off:.list.item.x:[0]"])
     array.layout.array.content.field("x").array
     assert set(array.caches[0].keys()) == set(
-        ["tmp:off:.list.item.x[0]", "tmp:col:.list.item.x[0]"]
+        ["tmp:off:.list.item.x:[0]", "tmp:col:.list.item.x[0]"]
     )
     assert array.tolist() == data
